/*
 ## Cypress USB 3.0 Platform source file (cyfxuvcdscr.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file contains the USB enumeration descriptors for the UVC (in memory) application example.
 * The descriptor arrays must be 32 byte aligned and multiple of 32 bytes if the D-cache is
 * turned on. If the linker used is not capable of supporting the aligned feature for this,
 * either the descriptors must be placed in a different section and the section should be 
 * 32 byte aligned and 32 byte multiple; or dynamically allocated buffer allocated using
 * CyU3PDmaBufferAlloc must be used, and the descriptor must be loaded into it. The example
 * assumes that the aligned attribute for 32 bytes is supported by the linker. Do not add
 * any other variables to this file other than USB descriptors. This is not the only
 * pre-requisite to enabling the D-cache. Refer to the documentation for
 * CyU3PDeviceCacheControl for more information.
 */

#include <cyu3utils.h>

#include "cyfxuvcinmem.h"

#if USB_UVC_CLASS == 1
#define UVC_DESC_LEN                    0xD1
#define UVC_INTF_NUM                    0x02
#else
#define UVC_DESC_LEN                    0x00
#define UVC_INTF_NUM                    0x00
#endif

#if USB_UAC_CLASS == 1
#define UAC_DESC_LEN                    0x69
#define UAC_INTF_NUM                    0x02
#else
#define UAC_DESC_LEN                    0x00
#define UAC_INTF_NUM                    0x00
#endif

#if USB_CDC_CLASS == 1
#define CDC_DESC_LEN                    0x54
#define CDC_INTF_NUM                    0x02
#else
#define CDC_DESC_LEN                    0x00
#define CDC_INTF_NUM                    0x00
#endif

/* Total size of descriptor. 0x09 comes from the top level Configuration Descriptor */
#define SS_TOTAL_SIZE_CONFIG_DSCR       (0x09 + UVC_DESC_LEN + UAC_DESC_LEN + CDC_DESC_LEN)
/* Low byte - Total size of the device descriptor */
#define SS_TOTAL_SIZE_CONFIG_DSCR_L     CY_U3P_GET_LSB(SS_TOTAL_SIZE_CONFIG_DSCR)
/* High byte - Total size of the device descriptor */
#define SS_TOTAL_SIZE_CONFIG_DSCR_H     CY_U3P_GET_MSB(SS_TOTAL_SIZE_CONFIG_DSCR)

/* Low Byte - UVC Video Streaming Endpoint Packet Size */
#define CY_FX_EP_BULK_VIDEO_PKT_SIZE_L  CY_U3P_GET_LSB(CY_FX_EP_BULK_VIDEO_PKT_SIZE)
/* High Byte - UVC Video Streaming Endpoint Packet Size */
#define CY_FX_EP_BULK_VIDEO_PKT_SIZE_H  CY_U3P_GET_MSB(CY_FX_EP_BULK_VIDEO_PKT_SIZE)

/* Total number of interfaces */
#define INTF_NUM                        (UVC_INTF_NUM + UAC_INTF_NUM + CDC_INTF_NUM)


/* UVC Class Codes */
#define CC_VIDEO                        (0x0E)

/* UVC Subclass Codes */
#define SC_UNDEFINED                    (0x00)
#define SC_VIDEOCONTROL                 (0x01)
#define SC_VIDEOSTREAMING               (0X02)
#define SC_VIDEO_INTERFACE_COLLECTION   (0X03)

/* UVC Protocol Codes */
#define PC_PROTOCOL_UNDEFINED           (0x00)
#define PC_PROTOCOL_15                  (0x01)

/* UVC Class-Specific Descriptor Types */
#define CS_UNDEFINED                    (0x20)
#define CS_DEVICE                       (0x21)
#define CS_CONFIGURATION                (0x22)
#define CS_STRING                       (0x23)
#define CS_INTERFACE                    (0x24)
#define CS_ENDPOINT                     (0x25)

/* UVC VideoControl Class-Specific Descriptor Subtypes */
#define VC_DESCRIPTOR_UNDEFINED         (0x00)
#define VC_HEADER                       (0x01)
#define VC_INPUT_TERMINAL               (0x02)
#define VC_OUTPUT_TERMINAL              (0x03)
#define VC_SELECTOR_UNIT                (0x04)
#define VC_PROCESSING_UNIT              (0x05)
#define VC_EXTENSION_UNIT               (0x06)
#define VC_ENCODING_UNIT                (0x07)

/* UVC VideoStreaming Class-Specific Descriptor Subtypes */
#define VS_UNDEFINED                    (0x00)
#define VS_INPUT_HEADER                 (0x01)
#define VS_OUTPUT_HEADER                (0x02)
#define VS_STILL_IMAGE_FRAME            (0x03)
#define VS_FORMAT_UNCOMPRESSED          (0x04)
#define VS_FRAME_UNCOMPRESSED           (0x05)
#define VS_FORMAT_MJPEG                 (0x06)
#define VS_FRAME_MJPEG                  (0x07)
#define VS_FORMAT_MPEG2TS               (0x0A)
#define VS_FORMAT_DV                    (0x0C)
#define VS_COLORFORMAT                  (0x0D)
#define VS_FORMAT_FRAME_BASED           (0x10)
#define VS_FRAME_FRAME_BASED            (0x11)
#define VS_FORMAT_STREAM_BASED          (0x12)
#define VS_FORMAT_H264                  (0x13)
#define VS_FRAME_H264                   (0x14)
#define VS_FORMAT_H264_SIMULCAST        (0x15)
#define VS_FORMAT_VP8                   (0x16)
#define VS_FRAME_VP8                    (0x17)
#define VS_FORMAT_VP8_SIMULCAST         (0x18)

/* UVC Class-Specific Endpoint Descriptor Subtypes */
#define EP_UNDEFINED                    (0x00)
#define EP_GENERAL                      (0x01)
#define EP_ENDPOINT                     (0x02)
#define EP_INTERRUPT                    (0x03)

/* Standard device descriptor for USB 3.0 */
const uint8_t CyFxUSB30DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x10,0x03,                      /* USB 3.1 */
    0xEF,                           /* Device class */
    0x02,                           /* Device Sub-class */
    0x01,                           /* Device protocol */
    0x09,                           /* Maxpacket size for EP0 : 2^9 bytes */
    0xB4,0x04,                      /* Vendor ID */
    0x22,0x47,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Standard device descriptor for USB 2.0*/
const uint8_t CyFxUSB20DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x10,0x02,                      /* USB 2.1 device. Value of 0x10,0x02 indicates support for BOS request */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0xB4,0x04,                      /* Vendor ID */
    0x22,0x47,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Binary device object store descriptor */
const uint8_t CyFxUSBBOSDscr[] __attribute__ ((aligned (32))) =
{
    0x05,                           /* Descriptor size */
    CY_U3P_BOS_DESCR,               /* Device descriptor type */
    0x16,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of device capability descriptors */

    /* USB 2.0 Extension */
    0x07,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_USB2_EXTN_CAPB_TYPE,     /* USB 2.0 extension capability type */
    0x1E,0x64,0x00,0x00,            /* Supported device level features: LPM support, BESL supported,
                                     * Baseline BESL=400 us, Deep BESL=1000 us. */

    /* SuperSpeed Device Capability */
    0x0A,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_SS_USB_CAPB_TYPE,        /* SuperSpeed device capability type */
    0x00,                           /* Supported device level features  */
    0x08,0x00,                      /* Speeds supported by the device : SS */
    0x03,                           /* Functionality support */
    0x00,                           /* U1 device exit latency */
    0x00,0x00                       /* U2 device exit latency */
};

/* Standard device qualifier descriptor */
const uint8_t CyFxUSBDeviceQualDscr[] __attribute__ ((aligned (32))) =
{
    0x0A,                           /* Descriptor Size */
    CY_U3P_USB_DEVQUAL_DESCR,       /* Device qualifier descriptor type */
    0x10,0x02,                      /* USB 2.0 w/ BOS support */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0x01,                           /* Number of configurations */
    0x00                            /* Reserved */
};

/* Standard super speed configuration descriptor */
const uint8_t CyFxUSBSSConfigDscr[] __attribute__ ((aligned (32))) =
{
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    SS_TOTAL_SIZE_CONFIG_DSCR_L,    /* Length of this descriptor and all sub descriptors */
    SS_TOTAL_SIZE_CONFIG_DSCR_H,
    INTF_NUM,                       /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* Configuration string index */
    0x80,                           /* Config characteristics - Bus powered */
    0x32,                           /* Max power consumption of device (in 8mA unit) : 400mA */

#if USB_UVC_CLASS == 1
    /*********************************************************************************************/
    /********************************UVC START len=0xD1******************************************/
    /*********************************************************************************************/
    /* Interface Association Descriptor (Video) */
    0x08,                           /* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    CY_FX_UVC_INTERFACE_VC,         /* I/f number of first VideoControl i/f */
    UVC_INTF_NUM,                   /* Number of video i/f */
    CC_VIDEO,                       /* Video i/f class code */
    SC_VIDEO_INTERFACE_COLLECTION,  /* Subclass code */
    PC_PROTOCOL_UNDEFINED,          /* Protocol : Not used */
    0x00,                           /* String desc index for interface */

    /* Standard VideoControl Interface Descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    CY_FX_UVC_INTERFACE_VC,         /* Interface number for this interface */
    CY_FX_UVC_INTERFACE_VC,         /* Alternate setting number
                                     * Rep. alternate setting this VC interface descriptor is for? */
    0x01,                           /* Number of end points, excluding EP0 */
    CC_VIDEO,                       /* Class code */
    SC_VIDEOCONTROL,                /* Subclass code */
    PC_PROTOCOL_UNDEFINED,          /* Interface protocol code: Unused */
    0x00,                           /* Interface descriptor string index
                                     * Must match IAD string index */

    /* Class-specific (VideoControl) Interface Header Descriptor */
    0x0D,                           /* Descriptor size */
    CS_INTERFACE,                   /* Descriptor type: Class-specific interface descriptor */
    VC_HEADER,                      /* Descriptor sub type*/
    0x10,0x01,                      /* Revision of class spec : 1.1 */
    0x51,0x00,                      /* Total size of class specific descriptors (till output terminal) */
    0x00,0x6C,0xDC,0x02,            /* Clock frequency : 48MHz (Deprecated) */
    0x01,                           /* Number of streaming interfaces belonging to this VC */
    CY_FX_UVC_INTERFACE_VS,         /* Video streaming i/f 1 belongs to VC i/f
                                     * List of interfaces which belong to the VC above? */

    /* Class-specific (VideoControl) Interface Descriptor - Input Terminal */
    0x12,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class specific interface desc type */
    VC_INPUT_TERMINAL,              /* Input Terminal Descriptor type */
    CY_FX_UVC_INPUT_TERMINAL_ID,    /* ID of this terminal */
    0x01,0x02,                      /* Camera terminal type */
    0x00,                           /* No association terminal */
    0x00,                           /* String desc index : Not used */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */

    /* Class-specific (VideoControl) Interface Descriptor - Processing Unit */
    0x0D,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class specific interface desc type */
    VC_PROCESSING_UNIT,             /* Processing unit descriptor type */
    CY_FX_UVC_PROCESSING_UNIT_ID,   /* ID of this terminal */
    CY_FX_UVC_INPUT_TERMINAL_ID,    /* Source ID : 1 : connected to input terminal */
    0x00,0x40,                      /* Digital multiplier */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : Not used */
    0x00,                           /* No analog mode support. */

    /* Class-specific (VideoControl) Interface Descriptor - Extension Unit */
    0x1C,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class specific interface desc type */
    VC_EXTENSION_UNIT,              /* Extension unit descriptor type */
    CY_FX_UVC_EXTENSION_UNIT_ID,    /* ID of this terminal */
    0xFF,0xFF,0xFF,0xFF,            /* 16 byte GUID. Unused, so left blank */
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0x00,                           /* Number of controls in this terminal */
    0x01,                           /* Number of input pins in this terminal */
    CY_FX_UVC_PROCESSING_UNIT_ID,   /* Source ID : 2 : connected to proc unit */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : Not used */

    /* Class-specific (VideoControl) Interface Descriptor - Output Terminal */
    0x09,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class specific interface desc type */
    VC_OUTPUT_TERMINAL,             /* Output Terminal Descriptor type */
    CY_FX_UVC_OUTPUT_TERMINAL_ID,   /* ID of this terminal */
    0x01,0x01,                      /* USB Streaming terminal type */
    0x00,                           /* No association terminal */
    CY_FX_UVC_EXTENSION_UNIT_ID,    /* Source ID : 3 : Connected to Extn Unit */
    0x00,                           /* String desc index : Not used */

    /* VideoControl Status Interrupt Endpoint Descriptor */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_CONTROL_STATUS,        /* Endpoint address and description */
    CY_U3P_USB_EP_INTR,             /* Interrupt end point type */
    0x40,0x00,                      /* Max packet size = 64 bytes */
    0x01,                           /* Servicing interval */

    /* SuperSpeed Endpoint Companion Descriptor */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
    0x00,                           /* Max no. of packets in a burst - 1 : 1 */
    0x00,                           /* Mult.: Max number of packets - 1 : 1 */
    0x00, 0x04,                     /* Bytes per interval : 1024 */

    /* Class-specific (VideoControl) Status Interrupt Endpoint Descriptor */
    0x05,                           /* Descriptor size */
    CS_ENDPOINT,                    /* Class specific endpoint descriptor type */
    CY_U3P_USB_EP_INTR,             /* End point sub type */
    0x40,0x00,                      /* Max packet size = 64 */

    /* Standard VideoStreaming Interface Descriptor (Interface 1, Alternate Setting 0) */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
    CY_FX_UVC_INTERFACE_VS,         /* Interface number */
    CY_FX_UVC_INTERFACE_VC,         /*? Alternate setting number */
    0x01,                           /* Number of end points */
    CC_VIDEO,                       /* Interface class : CC_VIDEO */
    SC_VIDEOSTREAMING,              /* Interface sub class : CC_VIDEOSTREAMING */
    PC_PROTOCOL_UNDEFINED,          /* Interface protocol code : Undefined */
    0x00,                           /* Interface descriptor string index */

    /* Class-specific (VideoStreaming) Input Header Descriptor */
    0x0E,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class-specific VS i/f Type */
    VS_INPUT_HEADER,                /* Descriptor subtype : input header */
    0x01,                           /* 1 format descriptor follows */
    0x47,0x00,                      /* Total size of class specific VS descr = 55 bytes */
    CY_FX_EP_BULK_VIDEO,            /* EP address for BULK video data */
    0x00,                           /* No dynamic format change supported */
    CY_FX_UVC_OUTPUT_TERMINAL_ID,   /* Output terminal ID : 4 */
    0x01,                           /* Still image capture support. */
    0x00,                           /* No hardware trigger support. */
    0x00,                           /* Hardware to initiate still image capture */
    0x01,                           /* Size of controls field : 1 byte */
    0x00,                           /* D2 : Compression quality not supported */

    /* Class-specific (VideoStreaming) Format (Uncompressed) Descriptor */
    0x1B,                           /* Descriptor size */
    CS_INTERFACE,                   /* Class-specific VS I/f Type */
    VS_FORMAT_UNCOMPRESSED,         /* Subtype : uncompressed format I/F */
    0x01,                           /* Format descriptor index */
    0x01,                           /* Number of frame descriptor followed */
    0x59,0x55,0x59,0x32,            /* GUID used to identify streaming-encoding format: YUY2(4:2:2 YUV)  */
    0x00,0x00,0x10,0x00,            /* packed 4:2:2YUV is YUY2, 4:2:0YUV is M420*/
    0x80,0x00,0x00,0xAA,            /* planar 4:2:0YUV is NV12/I420*/
    0x00,0x38,0x9B,0x71,
    0x10,                           /* Number of bits per pixel - 16 bits */
    0x01,                           /* Optimum Frame Index for this stream: 1 */
    0x10,                           /* X dimension of the picture aspect ratio; Non-interlaced */
    0x09,                           /* Y dimension of the picture aspect ratio: Non-interlaced */
    0x00,                           /* Interlace Flags: Progressive scanning, no interlace */
    0x00,                           /* duplication of the video stream restriction: 0 - no restriction */

    /* Class-specific (VideoStreaming) Frame (Uncompressed) Descriptor */
    0x1E,                           /* Descriptor size: 30 bytes */
    CS_INTERFACE,                   /* Class-specific VS i/f type */
    VS_FRAME_UNCOMPRESSED,          /* Subtype: uncompressed frame I/F */
    0x01,                           /* Frame descriptor index */
    0x01,                           /* Still image capture method 1 supported */
#if STREAM_720P_SELECT == 1
    0x00,0x05,                      /* Width of the frame : 1280 */
    0xD0,0x02,                      /* Height of the frame : 720 */
    0x00,0x00,0xBC,0x34,            /* Min bit rate bits/s */       /* 1280 * 720 * 16bit * 60fps */
    0x00,0x00,0xBC,0x34,            /* Max bit rate bits/s */
    0x00,0x20,0x1C,0x00,            /* Maximum video or still frame size in bytes */    /* 1280 * 720 * 2 */
#else
    0x80,0x07,                      /* Width of the frame : 1920 */
    0x38,0x04,                      /* Height of the frame : 1080 */
    0x00,0x00,0xA7,0x76,            /* Min bit rate bits/s */       /* 1920 * 1080 * 16bit * 60fps */
    0x00,0x00,0xA7,0x76,            /* Max bit rate bits/s */
    0x00,0x48,0x3F,0x00,            /* Maximum video or still frame size in bytes */    /* 1920 * 1080 * 2 */
#endif
//    0x15,0x16,0x05,0x00,            /* 30fps */                                         /*add,0x00051615=333333,1s/333333/100ns=30*/
    0x0A,0x8B,0x02,0x00,            /* Preferred frame rate: 60fps */                                         /*add,0x00051615=333333,1s/166666/100ns=60*/
    0x01,                           /* Frame interval type : 1 discrete setting. */
//    0x15,0x16,0x05,0x00,            /* 30fps */
    0x0A,0x8B,0x02,0x00,            /* 60fps */

    /* VideoStreaming Bulk Endpoint Descriptor */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_BULK_VIDEO,            /* Endpoint address and description */
    CY_U3P_USB_EP_BULK,             /* Bulk End Point */
    CY_FX_EP_BULK_VIDEO_PKT_SIZE_L, /* Maximum packet size: 1024 bytes */
    CY_FX_EP_BULK_VIDEO_PKT_SIZE_H,
    0x00,                           /* Servicing interval for data transfers */

    /* SuperSpeed Endpoint Companion Descriptor */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
//    0x00,                           /* Max no. of packets in a Burst - 1: 1 */
    0x0F,                           /* Max no. of packets in a Burst - 1: 16 */
    0x00,                           /* Mult.: Max number of packets : 1 */
    0x00,0x00,                      /* Field Valid only for Periodic Endpoints */
#endif /* USB_UVC_CLASS */

#if USB_UAC_CLASS == 1
    /*********************************************************************************************/
    /************************************UAC START len = 0x69*************************************/
    /*********************************************************************************************/
    /* Interface association descriptor */                                      /*add 20210823,usb Composite Device*/
    0x08,                           /* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    CY_FX_UAC_INTERFACE_VC,         /* I/f number of first VideoControl i/f */
    UAC_INTF_NUM,                   /* Number of video streaming i/f */
    0x01,                           /* CC_VIDEO : Video i/f class code */
    0x01,                           /* SC_VIDEO_INTERFACE_COLLECTION : Subclass code */
    0x00,                           /* Protocol : Not used */
    0x00,                           /* String desc index for interface */

    /* Standard Audio Control Interface Descriptor */
    0x09,                           /* Descriptor Size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor Type */
    CY_FX_UAC_INTERFACE_VC,         /* Interface number */                      /*add 20210823,modify 0x00 to 0x02*/
    0x00,                           /* Alternate setting */
    0x00,                           /* Number of endpoints - 0 endpoints */
    0x01,                           /* Interface Class - Audio */
    0x01,                           /* Interface SubClass - Audio Control */
    0x00,                           /* Interface Protocol - Unused */
    0x00,                           /* Interface string index */

    /* Class Specific Audio Control Interface Descriptor */
    0x09,                           /* Descriptor Size */
    0x24,                           /* Descriptor Type - CS_INTERFACE */
    0x01,                           /* Descriptor SubType - Header */
    0x00, 0x01,                     /* Revision of class specification - 1.0 */
    0x1E, 0x00,                     /* Total size of class specific descriptors */
    0x01,                           /* Number of streaming Interfaces - 1 */
//    0x01,                           /* Audio Streaming interface 1 belongs to this AudioControl Interface */
    0x03,                           /*add 20210827,this filed cause UAC enumeration fail*/

    /* Input terminal descriptor */
    0x0C,                           /* Descriptor size in bytes */
    0x24,                           /* CS Interface Descriptor */
    0x02,                           /* Input Terminal Descriptor subtype */
    0x01,                           /* ID Of the input terminal */
    0x01, 0x02,                     /* Microphone - terminal type */
    0x00,                            /* Association terminal - None */
    0x02,                            /* Number of channels - 2 */
    0x03, 0x00,                     /* spatial location of the logical channels - Front Left and Front Right */
    0x00,                            /* Channel names - Unused */
    0x00,                            /* String index for this descriptor - None */

    /* Output terminal descriptor */
    0x09,                            /* Descriptor size */
    0x24,                            /* Class specific interface desc type */
    0x03,                            /* Output terminal descriptor type */
    0x02,                            /* ID of this terminal */
    0x01, 0x01,                     /* Output terminal type: USB Streaming */
    0x00,                            /* Association terminal - Unused */
    0x01,                            /* Id of the terminal/unit to which this is connected - Input terminal id */
    0x00,                            /* String desc index : Not used */

    /* Standard Audio Streaming Interface Descriptor (Alternate setting 0) */
    0x09,                            /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x03,                            /* Interface number */                                    /*add 20210823,modify 0x01 to 0x03*/
    0x00,                            /* Alternate setting number */
    0x00,                            /* Number of end points : zero bandwidth */
    0x01,                            /* Interface class : Audio */
    0x02,                            /* Interface sub class : Audio Streaming */
    0x00,                            /* Interface protocol code : Unused */
    0x00,                            /* Interface descriptor string index */

    /* Standard Audio Streaming Interface descriptor (Alternate setting 1) */
    0x09,                            /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x03,                            /* Interface number */                                    /*add 20210823,modify 0x01 to 0x03*/
    0x01,                            /* Alternate setting number */
    0x01,                            /* Number of end points : 1 ISO EP */
    0x01,                            /* Interface Audio class */
    0x02,                            /* Interface Audio sub class - Audio Streaming */
    0x00,                            /* Interface protocol code : Unused */
    0x00,                            /* Interface descriptor string index */

    /* Class-specific Audio Streaming General Interface descriptor */
    0x07,                            /* Descriptor size */
    0x24,                            /* Class-specific AS i/f Type */
    0x01,                            /* Descriptotor subtype : AS General */
    0x02,                            /* Terminal Link - Output terminal id */
    0x01,                            /* Interface delay */
    0x01, 0x00,                     /* Audio data format - PCM */

    /* Class specific AS Format descriptor - Type I Format Descriptor */
    0x0B,                            /* Descriptor size */
    0x24,                            /* Class-specific Interface Descriptor Type */
    0x02,                            /* Format Type Descriptor subtype */
    0x01,                            /* PCM FORMAT_TYPE_I */
    0x02,                            /* Number of channels - 2 */
    0x02,                            /* Subframe size - 2 bytes per audio subframe */
    0x10,                            /* Bit resolution - 16 bits */
    0x01,                            /* Number of samping frequencies - 1 */
    0x80, 0xBB, 0x00,                /* Sampling frequency - 48000 Hz */

    /* Endpoint descriptor for ISO streaming Audio data */
    0x09,                            /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_ISO_AUDIO,             /* Endpoint address and description */
    0x05,                            /* ISO End point : Async */
    0x60, 0x00,                     /* Transaction size - 96 bytes */
    0x03,                            /* Servicing interval for data transfers */
    0x00,                            /* bRefresh */
    0x00,                            /* bSynchAddress */

    /* Super speed endpoint companion descriptor */
    0x06,                            /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,        /* SS endpoint companion descriptor type */
    0x00,                            /* Max no. of packets in a burst : 1 */
    0x00,                            /* Mult.: Max number of packets : 1 */
    0x60, 0x00,                     /* Bytes per interval : 1024 */

    /* Class Specific AS Isochronous Audio Data Endpoint Descriptor */
    0x07,                            /* Descriptor size in bytes */
    0x25,                            /* CS_ENDPOINT descriptor type */
    0x01,                            /* EP_GENERAL sub-descriptor type */
    0x00,                            /* bmAttributes - None    */
    0x00,                            /* bLockDelayUnits - Unused */
    0x00, 0x00,                        /* wLockDelay - unused */
#endif /* USB_UAC_CLASS */

#if USB_CDC_CLASS == 1
    /*********************************************************************************************/
    /***********************************CDC START len = 0x54**************************************/
    /*********************************************************************************************/
    /* Interface association descriptor */                            /*add 20210823,usb Composite Device*/
    0x08,                           /* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    CY_FX_CDC_INTERFACE_CMD,        /* I/f number of first CDC-ACM i/f */
    CDC_INTF_NUM,                   /* Number of CDC-ACM i/f */
    0x02,                           /* CDC class code */
    0x02,                           /* CDC : Subclass code */
    0x01,                           /* Protocol : Not used */
    0x00,                           /* String desc index for interface */

    /* Communication Interface descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
    CY_FX_CDC_INTERFACE_CMD,        /* Interface number */
    0x00,                           /* Alternate setting number */
    0x01,                           /* Number of endpoints */
    0x02,                           /* Interface class : Communication interface */
    0x02,                           /* Interface sub class */
    0x01,                           /* Interface protocol code */
    0x00,                           /* Interface descriptor string index */

    /* CDC Class-specific Descriptors */
    /* Header functional Descriptor */
    0x05,                           /* Descriptors length */
    0x24,                           /* Descriptor type : CS_Interface */
    0x00,                           /* DescriptorSubType : Header Functional Descriptor */
    0x10,0x01,                      /* bcdCDC : CDC Release Number */

    /* Abstract Control Management Functional Descriptor */
    0x04,                           /* Descriptors Length */
    0x24,                           /* bDescriptorType: CS_INTERFACE */
    0x02,                           /* bDescriptorSubType: Abstract Control Model Functional Descriptor */
    0x02,                           /* bmCapabilities: Supports the request combination of Set_Line_Coding,
                                       Set_Control_Line_State, Get_Line_Coding and the notification Serial_State */

    /* Union Functional Descriptor */
    0x05,                           /* Descriptors Length */
    0x24,                           /* bDescriptorType: CS_INTERFACE */
    0x06,                           /* bDescriptorSubType: Union Functional Descriptor */
    0x00,                           /* bMasterInterface */
    0x01,                           /* bSlaveInterface */

    /* Call Management Functional Descriptor */
    0x05,                           /*  Descriptors Length */
    0x24,                           /*  bDescriptorType: CS_INTERFACE */
    0x01,                           /*  bDescriptorSubType: Call Management Functional Descriptor */
    0x00,                           /*  bmCapabilities: Device sends/receives call management information
                                     *  only over the Communication Class Interface. */
    0x01,                           /*  Interface Number of Data Class interface */

    /* Endpoint Descriptor(Interrupt) */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_INTERRUPT,             /* Endpoint address and description */
    CY_U3P_USB_EP_INTR,             /* Interrupt endpoint type */
    0x40,0x00,                      /* Max packet size = 64 bytes */
    0x01,                           /* Servicing interval for data transfers */

    /* Super speed endpoint companion descriptor for interrupt endpoint */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
    0x00,                           /* Max no. of packets in a Burst : 1 */
    0x00,                           /* Mult.: Max number of packets : 1 */
    0x40,0x00,                      /* Bytes per interval : 64 */

    /* Data Interface Descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
    CY_FX_CDC_INTERFACE_DATA,       /* Interface number */
    0x00,                           /* Alternate setting number */
    0x02,                           /* Number of endpoints */
    0x0A,                           /* Interface class: Data interface */
    0x00,                           /* Interface sub class */
    0x00,                           /* Interface protocol code */
    0x00,                           /* Interface descriptor string index */

    /* Endpoint Descriptor(BULK-PRODUCER) */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_PRODUCER,              /* Endpoint address and description */
    CY_U3P_USB_EP_BULK,             /* BULK endpoint type */
    0x00,0x04,                      /* Max packet size = 1024 bytes */
    0x00,                           /* Servicing interval for data transfers */

    /* Super speed endpoint companion descriptor for producer ep */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
    0x00,                           /* Max no. of packets in a burst : 1 */
    0x00,                           /* Mult.: Max number of packets : 1 */
    0x00,0x00,                      /* Bytes per interval : 1024 */

    /* Endpoint Descriptor(BULK- CONSUMER) */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_CONSUMER,              /* Endpoint address and description */
    CY_U3P_USB_EP_BULK,             /* BULK endpoint type */
    0x00,0x04,                      /* Max packet size = 1024 bytes */
    0x00,                           /* Servicing interval for data transfers */

    /* Super speed endpoint companion descriptor for consumer ep */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
    0x00,                           /* Max no. of packets in a burst : 1 */
    0x00,                           /* Mult.: Max number of packets : 1 */
    0x00,0x00                       /* Bytes per interval : 1024 */
#endif /* USB_CDC_CLASS */
};

/* Standard High Speed Configuration Descriptor */
const uint8_t CyFxUSBHSConfigDscr[] __attribute__ ((aligned (32))) =
{
#if 0
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0xBE,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* Configuration string index */
    0x80,                           /* Config characteristics - bus powered */
    0xC8,                           /* Max power consumption of device (in 2mA unit) : 400mA */

    /* Interface association descriptor */
    0x08,                           /* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    0x00,                           /* I/f number of first video control i/f */
    0x02,                           /* Number of video streaming i/f */
    0x0E,                           /* CC_VIDEO : Video i/f class code */
    0x03,                           /* SC_VIDEO_INTERFACE_COLLECTION : subclass code */
    0x00,                           /* Protocol : not used */
    0x00,                           /* String desc index for interface */

    /* Standard video control interface descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x00,                           /* Interface number */
    0x00,                           /* Alternate setting number */
    0x01,                           /* Number of end points */
    0x0E,                           /* CC_VIDEO : Interface class */
    0x01,                           /* CC_VIDEOCONTROL : Interface sub class */
    0x00,                           /* Interface protocol code */
    0x00,                           /* Interface descriptor string index */

    /* Class specific VC interface header descriptor */
    0x0D,                           /* Descriptor size */
    0x24,                           /* Class Specific I/f header descriptor type */
    0x01,                           /* Descriptor sub type : VC_HEADER */
    0x10,0x01,                      /* Revision of class spec : 1.1 */
    0x51,0x00,                      /* Total size of class specific descriptors (till output terminal) */
    0x00,0x6C,0xDC,0x02,            /* Clock frequency : 48MHz */
    0x01,                           /* Number of streaming interfaces */
    0x01,                           /* Video streaming I/f 1 belongs to VC i/f */

    /* Input (camera) terminal descriptor */
    0x12,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x02,                           /* Input Terminal Descriptor type */
    0x01,                           /* ID of this terminal */
    0x01,0x02,                      /* Camera terminal type */
    0x00,                           /* No association terminal */
    0x00,                           /* String desc index : not used */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */

    /* Processing unit descriptor */
    0x0D,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x05,                           /* Processing unit descriptor type */
    0x02,                           /* ID of this terminal */
    0x01,                           /* Source ID : 1 : connected to input terminal */
    0x00,0x40,                      /* Digital multiplier */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : not used */
    0x00,                           /* No analog mode support. */

    /* Extension unit descriptor */
    0x1C,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x06,                           /* Extension unit descriptor type */
    0x03,                           /* ID of this terminal */
    0xFF,0xFF,0xFF,0xFF,            /* 16 byte GUID */
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0x00,                           /* Number of controls in this terminal */
    0x01,                           /* Number of input pins in this terminal */
    0x02,                           /* Source ID : 2 : connected to proc unit */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : not used */

    /* Output terminal descriptor */
    0x09,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x03,                           /* Output terminal descriptor type */
    0x04,                           /* ID of this terminal */
    0x01,0x01,                      /* USB Streaming terminal type */
    0x00,                           /* No association terminal */
    0x03,                           /* Source ID : 3 : connected to extn unit */
    0x00,                           /* String desc index : not used */

    /* Video control status interrupt endpoint descriptor */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_CONTROL_STATUS,        /* Endpoint address and description */
    CY_U3P_USB_EP_INTR,             /* Interrupt end point type */
    0x40,0x00,                      /* Max packet size = 64 bytes */
    0x08,                           /* Servicing interval : 8ms */

    /* Class specific interrupt endpoint descriptor */
    0x05,                           /* Descriptor size */
    0x25,                           /* Class specific endpoint descriptor type */
    CY_U3P_USB_EP_INTR,             /* End point sub type */
    0x40,0x00,                      /* Max packet size = 64 bytes */

    /* Standard video streaming interface descriptor (alternate setting 0) */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x01,                           /* Interface number */
    0x00,                           /* Alternate setting number */
    0x01,                           /* Number of end points : zero bandwidth */
    0x0E,                           /* Interface class : CC_VIDEO */
    0x02,                           /* Interface sub class : CC_VIDEOSTREAMING */
    0x00,                           /* Interface protocol code : undefined */
    0x00,                           /* Interface descriptor string index */

    /* Class-specific video streaming input header descriptor */
    0x0E,                           /* Descriptor size */
    0x24,                           /* Class-specific VS i/f type */
    0x01,                           /* Descriptor subtype : input header */
    0x01,                           /* 1 format desciptor follows */
    0x37,0x00,                      /* Total size of class specific VS descr = 55 bytes */
    CY_FX_EP_BULK_VIDEO,            /* EP address for BULK video data */
    0x00,                           /* No dynamic format change supported */
    0x04,                           /* Output terminal ID : 4 */
    0x00,                           /* No still image capture support. */
    0x00,                           /* No hardware trigger support. */
    0x00,                           /* Hardware to initiate still image capture */
    0x01,                           /* Size of controls field : 1 byte */
    0x00,                           /* D2 : Compression quality supported */

    /* Class specific VS format descriptor */
    0x0B,                           /* Descriptor size */
    0x24,                           /* Class-specific VS i/f type */
    0x06,                           /* Descriptor subtype : VS_FORMAT_MJPEG */
    0x01,                           /* Format desciptor index */
    0x01,                           /* 1 Frame desciptor follows */
    0x01,                           /* Uses fixed size samples */
    0x01,                           /* Default frame index is 1 */
    0x00,                           /* Non interlaced stream not reqd. */
    0x00,                           /* Non interlaced stream not reqd. */
    0x00,                           /* Non interlaced stream */
    0x00,                           /* CopyProtect: duplication unrestricted */

    /* Class specific VS frame descriptor */
    0x1E,                           /* Descriptor size */
    0x24,                           /* Class-specific VS I/f Type */
    0x07,                           /* Descriptor subtype : VS_FRAME_MJPEG */
    0x01,                           /* Frame desciptor index */
    0x00,                           /* Still image capture method not supported */
    0x80,0x02,                      /* Width of the frame  : 640 */
    0xE0,0x01,                      /* Height of the frame : 480 */
    0x00,0xC0,0x5D,0x00,            /* Min bit rate bits/s */
    0x00,0xC0,0x5D,0x00,            /* Max bit rate bits/s */
    0x00,0x58,0x02,0x00,            /* Maximum video or still frame size in bytes */
    0x2A,0x2C,0x0A,0x00,            /* Default frame interval */
    0x01,                           /* 1 Frame interval type : No of discrete intervals */
    0x2A,0x2C,0x0A,0x00,            /* Frame interval=15 fps */

    /* Endpoint descriptor for streaming video data */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_BULK_VIDEO,            /* Endpoint address and description */
    CY_U3P_USB_EP_BULK,             /* Bulk Endpoint */
    0x00, 0x02,                     /* 512 Bytes Maximum Packet Size. */
    0x00                            /* Servicing interval for data transfers */
#endif
};

/* Standard full speed configuration descriptor : full speed is not supported. */
const uint8_t CyFxUSBFSConfigDscr[] __attribute__ ((aligned (32))) =
{
#if 0
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x09,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x00,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - bus powered */
    0x32                            /* Max power consumption of device (in 2mA unit) : 100mA */
#endif
};

/* Standard language ID string descriptor */
const uint8_t CyFxUSBStringLangIDDscr[] __attribute__ ((aligned (32))) =
{
    0x04,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    0x09,0x04                       /* Language ID supported */
};

/* Standard manufacturer string descriptor */
const uint8_t CyFxUSBManufactureDscr[] __attribute__ ((aligned (32))) =
{
    0x10,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'C',0x00,
    'y',0x00,
    'p',0x00,
    'r',0x00,
    'e',0x00,
    's',0x00,
    's',0x00
};

/* Standard product string descriptor */
const uint8_t CyFxUSBProductDscr[] __attribute__ ((aligned (32))) =
{
    0x08,                           /* Descriptor Size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'F',0x00,
    'X',0x00,
    '3',0x00
};

/* Place this buffer as the last buffer so that no other variable / code shares
 * the same cache line. Do not add any other variables / arrays in this file.
 * This will lead to variables sharing the same cache line. */
const uint8_t CyFxUsbDscrAlignBuffer[32] __attribute__ ((aligned (32)));

/* [ ] */

