/*
 ## Cypress USB 3.0 Platform source file (cyfxuvcinmem.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file illustrates USB video class application example over BULK Endpoint (streaming from internal
   memory) 
 */

/*
   This example implements a USB video class Driver with the help of the appropriate USB enumeration
   descriptors over BULK Endpoint. With these descriptors, the FX3 device enumerates as a USB Video 
   Class device on the USB host.

   On successful enumeration the device shows up in the Windows Explorer. When the device is opened
   the host initiates a set of UVC specific class requests. The main class requests that need to be
   handled by the device are the GET/SET probe control request and SET commit control request.
   A predefined probe setting is returned as part of the Get Probe request. The Set probe / commit
   request is not interpreted and is only acknowledged.

   A successful set configuration starts the video streaming.

   The video streaming is accomplished with the help of a DMA MANUAL channel. Video frames are
   retrieved from an FPGA FIFO through GPIF. GPIF creates buffers using the FX3's DMA system and
   when a buffer is full, the CB attaches the appropriate UVC headers. With completion of each video
   frame the next indexed video frame is chosen for transfer. When all the frames are transferred,
   the index is reset to start transfer from the first video frame.

   CY_FX_UVC_STREAM_BUF_SIZE and CY_FX_UVC_STREAM_BUF_COUNT in the header file define the DMA buffer
   size and the number of DMA buffers respectively.

   This example is not supported on full speed interface.
 */


//#include <uart_regs.h>
//#include <math.h>

#include <cyu3system.h>      /* FX3 System Function Definitions */
#include <cyu3os.h>          /* FX3 OS Function Definitions */
#include <cyu3dma.h>         /* DMA Configuration APIs */
#include <cyu3error.h>       /* Error enumerations */
#include <cyu3usb.h>         /* USB Endpoint, Events, Setup APIs  */
#include <cyu3uart.h>        /* UART Configuration APIs */
#include <cyu3utils.h>       /* Macros CY_U3P_GET_MSB and CY_U3P_GET_LSB */
#include <cyu3pib.h>         /* PIB (P-Port) Configuration APIs */
#include <cyu3gpio.h>        /* GPIO Configuration APIs */
#include <cyu3socket.h>      /* DMA Socket Wrap-up */

#include "cyfxgpif2config.h" /* GPIF State Machine Configuration */
#include "cyfxuvcinmem.h"    /* Application Defines */

/** PIB Settings
 * Addressing:  No
 * Clocking:    External
 * Bus Width:   16-bit + UART
 * GPIF Interrupts on all states
 *
 * With internal GPIF clock:
 * Using SYS_CLK of 403.2 MHz and PIB clock divider of 4 results
 * - DMA left in error state
 * - State machine in UNKNOWN state
 * - 760 buffers "produced" (created by DMA channel)
 *
 * Add INTR_CPU in all states to track transitions
 *
 */

/********************************************************************************
 *               Variable Declarations                                          *
 ********************************************************************************/

uint64_t initErrors = 0; /* Bitmap representing specific initialization errors. Bits 0-31 are used */

#if USB_UVC_CLASS == 1 /* UVC Variables */

#if STREAM_720P_SELECT == 0
#define HD_FRAME_SIZE   (1920 * 1080 * 2)
#else
#define HD_FRAME_SIZE   (1280 * 720 * 2)
#endif

#if USE_DMA_OVERRIDE == 1
#define MOD             (HD_FRAME_SIZE / (CY_FX_UVC_STREAM_BUF_SIZE - CY_FX_UVC_MAX_HEADER)) /* 16KB - 12Byte */
#define REM             (HD_FRAME_SIZE % (CY_FX_UVC_STREAM_BUF_SIZE - CY_FX_UVC_MAX_HEADER))
#else
#define MOD             (HD_FRAME_SIZE / (CY_FX_UVC_STREAM_BUF_SIZE - 16)) /* 16KB - 16Byte */
#define REM             (HD_FRAME_SIZE % (CY_FX_UVC_STREAM_BUF_SIZE - 16)) /* Using DMA library, the footer 4 bits cannot be used for data */
#endif

static uint32_t mod = MOD; /* Number of full buffers */
static uint32_t rem = REM; /* How many bytes in the last buffer */

/*UVC uncompressed test*/
static CyU3PDmaBuffer_t UvcdmaBuffer[CY_FX_UVC_MAX_VID_FRAMES];

/* Setup data field : Request */
#define CY_U3P_USB_REQUEST_MASK                       (0x0000FF00)
#define CY_U3P_USB_REQUEST_POS                        (8)

/* UVC Header */
uint8_t glUVCHeader[CY_FX_UVC_MAX_HEADER] =
{
    0x0C,                           /* Header Length */
    0x8C,                           /* Bit field header field */
    0x00,0x00,0x00,0x00,            /* Presentation time stamp field */
    0x00,0x00,0x00,0x00,0x00,0x00   /* Source clock reference field */
};

/* Video Probe Commit Control */
uint8_t glCommitCtrl[CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED] __attribute__ ((aligned (32)));

CyU3PThread              uvcAppThread;                   /* USB streaming handler thread structure */
CyU3PThread              uvcAppEp0Thread;                /* USB EP0 control handler thread structure */
CyU3PThread              uvcAppProdThread;               /* DMA frame producer thread structure */

static CyU3PEvent        glFxUVCEvent;                   /* Event group used to signal threads. */

CyU3PDmaChannel          glChInterruptControl;           /* DMA Interrupt control handle */
CyU3PDmaChannel          glChHandleUVCStream;            /* DMA Channel Handle */
/* Below bools control program state. Due to use across threads, they are volatile */
static volatile CyBool_t glIsUVCDevConfigured = CyFalse; /* Whether SET_CONFIG is complete or not. */
static volatile CyBool_t glIsUVCApplnActive   = CyFalse; /* Whether the loopback application is active or not. */
static volatile CyBool_t glStreamingStarted   = CyFalse; /* Whether USB host has started streaming data */

volatile unsigned int glEnterIdle      = 0;
volatile unsigned int glEnterDss_state = 0;
volatile unsigned int glEnterWrite     = 0;
volatile unsigned int glEnterRead      = 0;
volatile unsigned int glEnterShort_pkt = 0;
volatile unsigned int glEnterZlp       = 0;
volatile unsigned int glEnterCallback  = 0;

/* Event counters to count data going through the FX3 */
uint32_t glEventProd = 0; /* Counts CY_U3P_DMA_CB_PROD_EVENT DMA events */
uint32_t glEventCons = 0; /* Counts CY_U3P_DMA_CB_CONS_EVENT DMA events */

/* Request data read from EP0 */
uint8_t  bRequest, bReqType;      /* Specific device request, Full request type data */
uint8_t  bType, bTarget;          /* Request type, Request recipient. Both extracted from bReqType */
uint16_t wValue, wIndex, wLength; /* Value, Index or offset, Number of bytes associated with request */

/* Mac OS does not send EP Clear feature when the app is closed. It just stops issuing IN tokens. So buffer commit failures
 * can be counted and if it reaches beyond a limit, streaming can be stopped. Buffer commit failure code can be cleared
 * on DMA Consumer event so that the limit is not reached under streaming conditions.  */
static uint8_t glCommitBufferFailureCount = 0;

/* Last error encountered. Sent to host if a error request is received. Not currently used */
static uint8_t glUVCErrorCode = CY_FX_UVC_VC_ERROR_CODE_NO_ERROR;

/* Variable to track whether the reason for DMA Reset is Frame Timer overflow or Commit Buffer Failure */
static uint8_t glDmaResetFlag = CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE;

/* Forward declarations */
static void CyFxUVCAddHeader(uint8_t *buffer_p, uint8_t frameInd);

#endif /* USB_UVC_CLASS */

#if USB_UAC_CLASS == 1 /* UAC Variables */

static CyU3PDmaBuffer_t UACDmaBuffer[2];

/* Setup data field : Request */
#define CY_U3P_USB_REQUEST_MASK                         (0x0000FF00)
#define CY_U3P_USB_REQUEST_POS                          (8)

/* Iscochronous Transfer length */
#define CY_FX3_ISO_XFER_LEN                             (96)
/* Give a timeout value of 5s for any flash programming. */
#define CY_FX_USB_SPI_TIMEOUT                           (5000)
/* Size in pages of the Audio sample to be streamed from the SPI Flash. */
#define CY_FX3_AUDIO_SAMPLE_SIZE_IN_PAGES               (0x3CD)

CyU3PThread uacAppThread;                              /* Thread structure */

CyU3PDmaChannel glUacStreamHandle;                     /* DMA Channel Handle  */
static volatile CyBool_t glIsUACApplnActive = CyFalse; /* Whether the UAC application is active or not. */
static volatile CyBool_t glIsDevConfigured = CyFalse;  /* Whether the USB device configuration is complete. */

#endif /* USB_UAC_CLASS */

#if USB_CDC_CLASS == 1 /* CDC Variables */

CyU3PThread       USBUARTAppThread;
CyU3PDmaChannel   glChHandleUsb2UartRecv;              /* DMA MANUAL receive channel handle.*/
CyU3PDmaChannel   glChHandleUsb2UartTrans;             /* DMA MANUAL transmit channel handle.*/
CyBool_t          glIsApplnActive = CyFalse;           /* Whether the application is active or not. */
CyU3PUartConfig_t glUartConfig = {0};                  /* Current UART configuration. */
uint8_t *USB2UARTResp;

/* CDC Class specific requests to be handled by this application. */
#define SET_LINE_CODING         0x20
#define GET_LINE_CODING         0x21
#define SET_CONTROL_LINE_STATE  0x22

#define FRAME_SIZE              64

#endif /* USB_CDC_CLASS */

/********************************************************************************
 *               Common Functions                                               *
 ********************************************************************************/

#if USE_INTERRUPT_LOGS == 1
/* Send a interrupt packet to the host containing up to 8 hex characters.
 * Used for debugging since I dont have console log access */
void
CyFxSendStatusToHost(uint64_t input, CyBool_t isLog)
{
    CyU3PDmaBuffer_t StatusBuffer;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint8_t glStatusBuffer[16];

    glStatusBuffer[0] = 0x02;                     /* bStatusType = VideoStreaming interface */
    glStatusBuffer[1] = CY_FX_EP_CONTROL_SOCKET;  /* bOriginator = ID of VideoStreaming interface */
    if (isLog == CyTrue)                          /* Mark if input is error or log */
        glStatusBuffer[2] = 0x01;                 /* 0x01 = Log, 0x02 = Error */
    else
        glStatusBuffer[2] = 0x02;
    glStatusBuffer[3]  = (input >> 56) & 0xFF;    /* bValue      = Input I want to send... */
    glStatusBuffer[4]  = (input >> 48) & 0xFF;    /* Sent as big endian */
    glStatusBuffer[5]  = (input >> 40) & 0xFF;
    glStatusBuffer[6]  = (input >> 32) & 0xFF;
    glStatusBuffer[7]  = (input >> 24) & 0xFF;
    glStatusBuffer[8]  = (input >> 16) & 0xFF;
    glStatusBuffer[9]  = (input >>  8) & 0xFF;
    glStatusBuffer[10] =  input        & 0xFF;

    StatusBuffer.buffer = glStatusBuffer;
    StatusBuffer.count  = 11;
    StatusBuffer.size   = sizeof(glStatusBuffer);
    StatusBuffer.status = 0;

    apiRetStatus = CyU3PDmaChannelSetupSendBuffer(&glChInterruptControl, &StatusBuffer);
    if (apiRetStatus != CY_U3P_SUCCESS)
        CyU3PDebugPrint(2, "SetupSendBuffer failed with 0x%x\r\n", apiRetStatus);

    apiRetStatus = CyU3PDmaChannelWaitForCompletion(&glChInterruptControl, CYU3P_WAIT_FOREVER);
    if (apiRetStatus != CY_U3P_SUCCESS)
        CyU3PDebugPrint(2, "WaitForCompletion failed with 0x%x\r\n",apiRetStatus);
}

/* Send an interrupt message to host marked as either a log or error. Allows up to 16 hex characters
 * to be sent per log */
#define SEND_INTERRUPT_LOG(input)   CyFxSendStatusToHost(input, CyTrue)
#define SEND_INTERRUPT_ERR(input)   CyFxSendStatusToHost(input, CyFalse)

#else
/* Remove logs from firmware */
#define SEND_INTERRUPT_LOG(input)
#define SEND_INTERRUPT_ERR(input)

#endif

/* Print the current configuration of the application. */
static void
CyFxApplnPrintConf (void)
{
    CyU3PDebugPrint(4, "Application Configuration:\r\n");

    CyU3PDebugPrint(4, " UVC Status: %d\r\n", USB_UVC_CLASS);
    CyU3PDebugPrint(4, " UAC Status: %d\r\n", USB_UAC_CLASS);
    CyU3PDebugPrint(4, " CDC Status: %d\r\n", USB_CDC_CLASS);

    CyU3PDebugPrint(4, " DMA Override: %d\r\n", USE_DMA_OVERRIDE);
    CyU3PDebugPrint(4, " Interrupt Logs: %d\r\n", USE_INTERRUPT_LOGS);
    CyU3PDebugPrint(4, " Data Bus: %d-bit\r\n", (GPIF_16BIT_CONF_SELECT == 1 ? 16 : 32));
    CyU3PDebugPrint(4, " Streaming Res: %dp60\r\n", (STREAM_720P_SELECT == 1) ? 720 : 1080);
    CyU3PDebugPrint(4, " Init Errors: 0x%x\r\n", initErrors);

    CyU3PDebugPrint(4, "\r\n");
}

/* This function performs the operations for a VideoStreaming abort.
 * This is called every time there is a USB reset, suspend or disconnect event. */
static void
CyFxUVCApplnAbortHandler (
        void)
{
    /* Set Video Stream Abort Event. The main UVC app thread will see this event and trigger a stream abort */
    CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_STREAM_ABORT_EVENT, CYU3P_EVENT_OR);
}

/* Application error handler. Logs that an error occurred and then goes into an
 * infinite loop. More specific error reporting code is done by
 * CyFxAppErrorHandler's caller because it has more context. */
void
CyFxAppErrorHandler (
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */
    CyU3PDebugPrint (4, "CyFxAppErrorHandler...\r\n");

    /* Loop indefinitely */
    for (;;)
    {
        /* Thread sleep : 100 ms */
        CyU3PThreadSleep (100);
    }
}

/* Fills out a frame buffer with a provided value. Debug function. Freezes FX3
 * if called at certain parts of the firmware, such as any _ThreadEntry functions.
 * Not sure why. */
void
CyFxAppFillBuffer (
        CyU3PDmaBuffer_t *dmaBuffer,
        int value)
{
    uint16_t i = 0;

    dmaBuffer->buffer = (uint8_t *)CyU3PDmaBufferAlloc(CY_FX_UVC_STREAM_BUF_SIZE);  /*add,16KB*/

    CyFxUVCAddHeader(dmaBuffer->buffer, CY_FX_UVC_HEADER_FRAME);

    for(i = 0;i < (CY_FX_UVC_STREAM_BUF_SIZE - CY_FX_UVC_MAX_HEADER);i++)
    {
        dmaBuffer->buffer[CY_FX_UVC_MAX_HEADER + i] = value;
    }
}


/********************************************************************************
 *               GPIF II Functions                                              *
 ********************************************************************************/

/* GpifCB callback function is invoked when FV triggers GPIF interrupt. Used to
 * wrap up incomplete packets and zero length packets.
 *
 * Do not use debug prints within callback function. */
void
CyFxGpifCB(
        uint8_t currentState)
{
    glEnterCallback++;

    /* Note: API calls which require mutexes cannot be directly called in this context */
    switch (currentState)
    {
    case RESET:
        break;
    case IDLE:
        /* IN_ADDR */
        glEnterIdle++;
        break;
    case READ:      /* Reads data from socket and drives data bus - SHOULDNT BE POSSIBLE */
        /* DR_DATA */
        /* IN_ADDR */
        glEnterRead++;
        break;
    case WRITE:     /* A full packet is produced by GPIF */
        /* IN_DATA */
        /* IN_ADDR */
        glEnterWrite++;
        break;
    case SHORT_PKT: /* A partially complete packet is produced by GPIF */
        /* COMMIT */
        /* IN_DATA */
        /* IN_ADDR */
        glEnterShort_pkt++;
        CyU3PDmaSocketSetWrapUp(CY_FX_EP_VIDEO_PROD_SOCKET); /* Seems to cause issues with host when used incorrectly. Disabled for now */
        break;
    case ZLP:       /* Just PKEND is asserted - An empty packet */
//        CyU3PDmaSocketSetWrapUp(CY_FX_EP_VIDEO_PROD_SOCKET); /* Seems to cause issues with host when used incorrectly. Disabled for now */
        /* COMMIT */
        /* IN_ADDR */
        glEnterZlp++;
        break;
    case DSS_STATE: /* Used to help differentiate states in GPIF. Required for the GPIF algorithm to generate the state machine */
        /* IN_ADDR */
        glEnterDss_state++;
        break;

    default:
        /* This should not happen. Do nothing. */
        return;
    }
}

/* Load the GPIF configuration on the GPIF-II engine. This operation is performed
 * at startup, before any threads are created.
 *
 * TODO: Should this run before or after main UVC thread? */
CyU3PReturnStatus_t
CyFxApplnGpifInit(void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Load GPIF state machine */
    apiRetStatus = CyU3PGpifLoad((CyU3PGpifConfig_t *) &CyFxGpifConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "Loading GPIF configuration failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 29;
        return apiRetStatus;
    }

#if USE_DMA_OVERRIDE != 1 /* In DMA override mode there is no watermarking */
    /* Configure active socket + watermarks */
    apiRetStatus = CyU3PGpifSocketConfigure(
        0,                          /* Thread to configure */
        CY_FX_EP_VIDEO_PROD_SOCKET, /* Socket to associate with this thread */
        253,                        /* Watermark value in 4-byte words - 1024 bytes max ~144 for audio when we get there~ */
        CyFalse,                    /* Whether watermark flag should be "set" (read "high") when above watermark */
        1                           /* Burst size (X * 2 words). Affects how often watermark is updated */
    );
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "GPIF socket configuration failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 30;
        return apiRetStatus;
    }
#endif

    CyU3PDebugPrint(4, "GPIF configuration initialized\r\n");

    return apiRetStatus;
}

/* Start the GPIF state machine from the reset state. Should be called during device
 * configuration.
 * NOT called every time streaming starts! */
CyU3PReturnStatus_t
CyFxApplnGpifStart(void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* NOTE: FX3 is the slave between the FX3-FPGA system so CyU3PGpifSMStart and thus if
     * deactivated or stopped the state should reset since FX3 doesnt maintain state from
     * last stream.
     * TODO: Verify */
    apiRetStatus = CyU3PGpifSMStart(RESET, ALPHA_RESET);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "CyU3PGpifSMStart failed, Error code = 0x%x\n", apiRetStatus);
        initErrors |= 1 << 31;
        return apiRetStatus;
    }

    CyU3PDebugPrint(4, "GPIF started\r\n");
    return apiRetStatus;
}

/* Stop the GPIF state machine. Performed when the device is unplugged or the
 * streaming is restarted. After calling this function the device should not be
 * used in streaming until the state machine has been restarted with
 * CyFxApplnGpifStart.
 * NOT called every time streaming stops! */
CyU3PReturnStatus_t
CyFxApplnGpifStop(void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Stops the state machine so it can be restarted from the reset state.
     * NOTE: CyU3PGpifDisable;s parameter should be CyFalse so state machine can be restarted with
     * CyU3PGpifSMStart in CyFxApplnGpifStart. */
    CyU3PGpifDisable(CyFalse);

    CyU3PDebugPrint(4, "GPIF stopped\r\n");
    return apiRetStatus;
}

/* Return a string version of the GPIF state.
 * Returns UNKNOWN if the state is not recognized. */
static inline char*
getGpifStateString(uint8_t state)
{
    switch (state)
    {
    case RESET:
        return "RESET";
    case IDLE:
        return "IDLE";
    case READ:
        return "READ";
    case WRITE:
        return "WRITE";
    case SHORT_PKT:
        return "SHORT_PKT";
    case ZLP:
        return "ZLP";
    case DSS_STATE:
        return "DSS_STATE";
    case 255:
        return "INVALID";
    case 19:
        return "19";
    case 18:
        return "18";
    case 17:
        return "17";
    case 16:
        return "16";
    case 15:
        return "15";
    case 14:
        return "14";
    case 13:
        return "13";
    case 12:
        return "12";
    case 11:
        return "11";
    case 10:
        return "10";
    case 9:
        return "09";
    case 8:
        return "08";
    case 7:
        return "07";
    default:
        return "UNKNOWN";
    }
//    return (state == RESET)     ? "RESET" :
//           (state == IDLE)      ? "IDLE" :
//           (state == READ)      ? "READ" :
//           (state == WRITE)     ? "WRITE" :
//           (state == SHORT_PKT) ? "SHORT_PKT" :
//           (state == ZLP)       ? "ZLP" :
//           (state == DSS_STATE) ? "DSS_STATE" :
//           (state == 255)       ? "INVALID STATE (255)" :
//           (state > 200)        ? "> 200" :
//           (state > 180)        ? "> 180" :
//           (state > 160)        ? "> 160" :
//           (state > 140)        ? "> 140" :
//           (state > 120)        ? "> 120" :
//           (state > 100)        ? "> 100" :
//           (state > 80)         ? "> 80" :
//           (state > 60)         ? "> 60" :
//           (state > 40)         ? "> 40" :
//           (state > 20)         ? "> 20" :
//                                  "< 20";
}

/********************************************************************************
 *               DMA Functions                                                  *
 ********************************************************************************/

/* DMA callback providing notification when data buffers are received by the FPGA FIFO or consumed by
 * the USB host.
 *
 * The UVC headers are attached to the data, and forwarded to the USB host in this
 * callback function.
 *
 * Do not use debug prints within callback function.
 */
void
CyFxApplnDmaCB (
        CyU3PDmaChannel   *chHandle,
        CyU3PDmaCbType_t   type,
        CyU3PDmaCBInput_t *input
        )
{
    CyU3PDmaBuffer_t    dmaBuffer;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    CyU3PEventSet(&glFxUVCEvent, CY_FX_UVC_DMA_CB, CYU3P_EVENT_OR);

    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
        /* This is a produce event notification to the CPU. This notification is received upon reception of
         * every buffer. The buffer will not be sent out unless it is explicitly committed. The call shall fail
         * if there is a bus reset / usb disconnect or if there is any application error. */
        glEventProd++;

        /* Get a buffer containing data from producer (P-port)
         * There is a possibility that CyU3PDmaChannelGetBuffer will return CY_U3P_ERROR_INVALID_SEQUENCE here.
         * In such a case, do nothing. We make up for this missed produce event by making repeated commit actions
         * in subsequent produce event callbacks.
         */
        apiRetStatus = CyU3PDmaChannelGetBuffer(chHandle, &dmaBuffer, CYU3P_NO_WAIT);
        while (apiRetStatus == CY_U3P_SUCCESS)
        {
            /* Add headers */
            if (dmaBuffer.count == CY_FX_UVC_BUF_FULL_SIZE)
            {
                /* A full buffer indicates there is more video frame packets to come. */
                CyFxUVCAddHeader(dmaBuffer.buffer - CY_FX_UVC_MAX_HEADER, CY_FX_UVC_HEADER_FRAME);
            }
            else
            {
                /* A partially filled buffer indicates the end of the ongoing video frame. */
                CyFxUVCAddHeader(dmaBuffer.buffer - CY_FX_UVC_MAX_HEADER, CY_FX_UVC_HEADER_EOF);
            }

            /* Commit buffer to USB */
            apiRetStatus = CyU3PDmaChannelCommitBuffer(chHandle, (dmaBuffer.count + CY_FX_UVC_MAX_HEADER), 0);
            if (apiRetStatus != CY_U3P_SUCCESS)
            {
                /* Problem committing buffer - set error if not already done and break */
                if(glDmaResetFlag == CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE)
                {
                    glDmaResetFlag = CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE;
                    CyU3PEventSet(&glFxUVCEvent, CY_FX_UVC_DMA_RESET_EVENT, CYU3P_EVENT_OR);
                }
                break;
            }

            /* Check for more buffers */
            apiRetStatus = CyU3PDmaChannelGetBuffer(chHandle, &dmaBuffer, CYU3P_NO_WAIT);
        }
    }
    else if (type == CY_U3P_DMA_CB_CONS_EVENT)
    {
        /* DMA consumption event */
        glEventCons++;
        glStreamingStarted = CyTrue;           /* Track that data is being consumed */
        glCommitBufferFailureCount = 0;        /* Reset the counter after data is consumed by USB */
    }
}

/* Return a string version of a DMA status enum.
 * Returns UNKNOWN if the state is not recognized. */
static inline char*
getDmaStateString(CyU3PDmaState_t state)
{
    return (state == CY_U3P_DMA_NOT_CONFIGURED) ? "CY_U3P_DMA_NOT_CONFIGURED" :
           (state == CY_U3P_DMA_CONFIGURED)     ? "CY_U3P_DMA_CONFIGURED" :
           (state == CY_U3P_DMA_ACTIVE)         ? "CY_U3P_DMA_ACTIVE" :
           (state == CY_U3P_DMA_PROD_OVERRIDE)  ? "CY_U3P_DMA_PROD_OVERRIDE" :
           (state == CY_U3P_DMA_CONS_OVERRIDE)  ? "CY_U3P_DMA_CONS_OVERRIDE" :
           (state == CY_U3P_DMA_ERROR)          ? "CY_U3P_DMA_ERROR" :
           (state == CY_U3P_DMA_IN_COMPLETION)  ? "CY_U3P_DMA_IN_COMPLETION" :
           (state == CY_U3P_DMA_ABORTED)        ? "CY_U3P_DMA_ABORTED" :
           (state == CY_U3P_DMA_NUM_STATES)     ? "CY_U3P_DMA_NUM_STATES" :
           (state == CY_U3P_DMA_XFER_COMPLETED) ? "CY_U3P_DMA_XFER_COMPLETED" :
           (state == CY_U3P_DMA_SEND_COMPLETED) ? "CY_U3P_DMA_SEND_COMPLETED" :
           (state == CY_U3P_DMA_RECV_COMPLETED) ? "CY_U3P_DMA_RECV_COMPLETED" :
                                                  "UNKNOWN";
}

/********************************************************************************
 *               Application Functions                                          *
 ********************************************************************************/

#if USB_UVC_CLASS == 1

/* Adds a UVC header onto a DMA buffer, overwriting the first 12 bytes of the
 * buffer. Therefore actual UVC data must start on the 13th byte. */
static void
CyFxUVCAddHeader(
        uint8_t *buffer_p, /* Buffer pointer */
        uint8_t frameInd   /* EOF or normal frame indication */
    )
{
    /* Copy header to buffer */
    CyU3PMemCopy(buffer_p, (uint8_t *)glUVCHeader, CY_FX_UVC_MAX_HEADER);

    /* Check if last packet of the frame. */
    if (frameInd == CY_FX_UVC_HEADER_EOF)
    {
        /* Toggle Frame ID in header struct. We are using uncompressed which is frame based, so switch bit for next frame */
        glUVCHeader[1] ^= CY_FX_UVC_HEADER_FRAME_ID;

        /* Indicate End of Frame in this buffer */
        buffer_p[1] |=  CY_FX_UVC_HEADER_EOF;
    }
}

/* This function starts the video streaming application. It is called
 * when there is a SET_INTERFACE event for alternate interface 1.
 * NOT called every time streaming is requested by the host. */
CyU3PReturnStatus_t
CyFxUVCApplnStart (void)
{
    CyU3PEpConfig_t         epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PReturnStatus_t     apiRetStatus = CY_U3P_SUCCESS;
    CyU3PUSBSpeed_t         usbSpeed     = CyU3PUsbGetSpeed();

    if (usbSpeed == CY_U3P_SUPER_SPEED)
    {
        CyU3PDebugPrint(4, "CY_U3P_SUPER_SPEED mode\r\n");
    }

    /* VideoStreaming USB IN Endpoint Configuration
     * TODO: Should this EP be created in Init and then activated in Start instead of being
     * setup and torn down every Start / Stop cycle? */
    epCfg.enable   = CyTrue;
    epCfg.epType   = CY_U3P_USB_EP_BULK;
    epCfg.pcktSize = CY_FX_EP_BULK_VIDEO_PKT_SIZE;
    epCfg.isoPkts  = 0; /* Bulk, so not needed? */
    epCfg.burstLen = CY_FX_EP_BULK_VIDEO_PKTS_COUNT;
    epCfg.streams  = 0;
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_BULK_VIDEO, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "UVC start CyU3PSetEpConfig failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 22;
        return apiRetStatus;
    }

    dmaCfg.size           = CY_FX_UVC_STREAM_BUF_SIZE;  /* 16KB */
#if USE_DMA_OVERRIDE == 1
    dmaCfg.count          = 0;                          /* Buffers are manually inserted using DMA override and sent in main loop */
#else
    dmaCfg.count          = CY_FX_UVC_STREAM_BUF_COUNT; /* Buffers are created + filled by DMA library API's CyU3PDmaChannelGetBuffer */
#endif
    dmaCfg.prodSckId      = CY_FX_EP_VIDEO_PROD_SOCKET;
    dmaCfg.consSckId      = CY_FX_EP_VIDEO_CONS_SOCKET; /* add,0x0301 */
    dmaCfg.dmaMode        = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.prodAvailCount = 0;
#if USE_DMA_OVERRIDE == 1
    dmaCfg.prodHeader     = 0; /* Header is inserted in the proper spot and the data fills around it */
    dmaCfg.prodFooter     = 0;
    dmaCfg.consHeader     = 0;
    dmaCfg.cb             = NULL;
    /* MANUAL_OUT connects CPU_PROD socket to any CONS socket */
    apiRetStatus = CyU3PDmaChannelCreate(&glChHandleUVCStream, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
#else
    dmaCfg.prodHeader     = 12;
    dmaCfg.prodFooter     = 4;  /* Buffer size must be 16 byte multiple, so 16 byte - 12 byte header = 4 byte leftover */
    dmaCfg.consHeader     = 0;
    dmaCfg.dmaMode        = CY_U3P_DMA_MODE_BYTE;       /* Added - working? */
    dmaCfg.notification   = CY_U3P_DMA_CB_PROD_EVENT |  /* Added - working? */
                            CY_U3P_DMA_CB_CONS_EVENT ;
    dmaCfg.cb             = CyFxApplnDmaCB;
    /* MANUAL connects any PROD socket to any CONS socket */
    apiRetStatus = CyU3PDmaChannelCreate(&glChHandleUVCStream, CY_U3P_DMA_TYPE_MANUAL, &dmaCfg);
#endif
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "UVC start CyU3PDmaChannelCreate failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 23;
        return apiRetStatus;
    }

    if (dmaCfg.count != 0)
    {
        /* Flush the USB IN Endpoint memory */
        CyU3PUsbFlushEp(CY_FX_EP_BULK_VIDEO);

        /* Set up for infinite transfer */
        apiRetStatus = CyU3PDmaChannelSetXfer(&glChHandleUVCStream, 0);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            CyU3PDebugPrint(4, "CyU3PDmaChannelSetXfer failed, Error code = 0x%x\r\n", apiRetStatus);
            initErrors |= 1 << 24;
            return apiRetStatus;
        }
    }

    /* Reset DMA flag */
    glDmaResetFlag = CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE;

    /* Start GPIF state machine */
    CyFxApplnGpifStart();

    /* Update the active state so the threads are aware UVC streaming is active. */
    glIsUVCApplnActive = CyTrue;

    CyU3PDebugPrint(3, "UVC App Started\r\n");

    return CY_U3P_SUCCESS;
}

/* This function stops the video streaming. It is called from the USB event
 * handler, when there is a reset / disconnect or SET_INTERFACE for alternate
 * interface 0.
 * NOT called every time streaming stops. */
void
CyFxUVCApplnStop (void)
{
    CyU3PEpConfig_t epCfg;

    /* Update the flag so that the application thread is notified of this. */
    glStreamingStarted = CyFalse;
    glDmaResetFlag = CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE;

    /* Stop GPIF state machine */
    CyFxApplnGpifStop();

    /* Abort and destroy the video streaming channel */
    CyU3PDmaChannelDestroy(&glChHandleUVCStream);

    /* Flush the endpoint memory - done on stream start instead */
    //CyU3PUsbFlushEp(CY_FX_EP_BULK_VIDEO);

    /* Disable the VideoStreaming USB IN Endpoint. */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyFalse;
    CyU3PSetEpConfig(CY_FX_EP_BULK_VIDEO, &epCfg);

    CyU3PDebugPrint(3, "UVC App Stopped\r\n");
}

/* Callback function to handle USB events.
 *
 * Do not use debug prints within callback function. */
static void
CyFxUVCApplnUSBEventCB (
    CyU3PUsbEventType_t evtype, /* Event type */
    uint16_t            evdata  /* Event data */
    )
{
    switch (evtype)
    {
    case CY_U3P_USB_EVENT_SETCONF:
        if (CyU3PUsbGetSpeed() == CY_U3P_SUPER_SPEED)
        {
            CyU3PDebugPrint(4, "UsbEventCB: Detected SS USB Connection\r\n");
        }
        else if (CyU3PUsbGetSpeed() == CY_U3P_HIGH_SPEED)
        {
            CyU3PDebugPrint(4, "UsbEventCB: Detected HS USB Connection\r\n");
        }
        /* Can control this event externally so print configuration on plug-in */
        CyFxApplnPrintConf();
        /* Fall-through */
    case CY_U3P_USB_EVENT_RESET:
    case CY_U3P_USB_EVENT_DISCONNECT:
    case CY_U3P_USB_EVENT_CONNECT:
        if (evtype == CY_U3P_USB_EVENT_SETCONF)
            glIsUVCDevConfigured = CyTrue;
        else
            glIsUVCDevConfigured = CyFalse;

        /* Stop the video streamer application. */
        if (glIsUVCApplnActive)
        {
            CyU3PDebugPrint(4, "UsbEventCB: Call App Stop\r\n");
            CyFxUVCApplnAbortHandler();
        }
        break;

    default:
        break;
    }
}

/* Handles UVC requests targeting the control interface.
 *
 * Eventually CyFxUVCApplnUSBSetupCB should signal the EP0 thread with an event and then EP0
 * calls the handler. That way it is easier to add handlers and keep them organized. */
static void
CyFxUVCHandleControlRequest (
        void)
{
    uint8_t temp;

    /* Currently only Error code requests are supported */
    switch (wValue)
    {
    case CY_FX_UVC_VC_REQUEST_ERROR_CODE_CONTROL:
        switch (bRequest)
        {
            case CY_FX_USB_UVC_GET_CUR_REQ:
                temp = CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL; /* Temporarily return no error. Replace 'temp' with a global var */
                CyU3PUsbSendEP0Data(1, &temp);
                break;
        }
        break;

    default:
        /* Unsupported request. Fail by stalling the control endpoint. */
        CyU3PUsbStall (0, CyTrue, CyFalse);
        break;
    }
}

/* Handles UVC requests targeting the streaming interface.
 *
 * Eventually CyFxUVCApplnUSBSetupCB should signal the EP0 thread with an event and then EP0
 * calls the handler. That way it is easier to add handlers and keep them organized. */
static void
CyFxUVCHandleStreamingRequest (
        void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint8_t ep0Buffer[32];
    uint16_t readCount = 0;

    switch (wValue)
    {
    case CY_FX_USB_UVC_VS_PROBE_CONTROL:
    case CY_FX_USB_UVC_VS_COMMIT_CONTROL:
        /* Negotiation requests from host. One streaming setting is supported so
         * Probe and Commit are the same, except that Commit actually starts
         * the streaming */
        switch (bRequest)
        {
        case CY_FX_USB_UVC_GET_INFO_REQ:
            ep0Buffer[0] = 3;                /* GET/SET requests are supported. */
            CyU3PUsbSendEP0Data (1, (uint8_t *)ep0Buffer);
            break;
        case CY_FX_USB_UVC_GET_LEN_REQ:
            ep0Buffer[0] = CY_FX_UVC_MAX_PROBE_SETTING;
            CyU3PUsbSendEP0Data (1, (uint8_t *)ep0Buffer);
            break;
        /* We only have one functional setting. Keep returning the same as current, default
         * minimum and maximum. */
        case CY_FX_USB_UVC_GET_CUR_REQ:
        case CY_FX_USB_UVC_GET_DEF_REQ:
        case CY_FX_USB_UVC_GET_MIN_REQ:
        case CY_FX_USB_UVC_GET_MAX_REQ:
            apiRetStatus = CyU3PUsbSendEP0Data(CY_FX_UVC_MAX_PROBE_SETTING,
                    (uint8_t *)glProbeCtrl);
            if (apiRetStatus != CY_U3P_SUCCESS)
            {
                initErrors |= 1 << 25;
//                CyU3PDebugPrint(4, "GET request CyU3PUsbSendEP0Data, Error code = 0x%x\n", apiRetStatus);
            }
            break;

        case CY_FX_USB_UVC_SET_CUR_REQ:
            /* Disable the low power entry to optimize USB throughput */
            CyU3PUsbLPMDisable();

            /* Read the data out into a local buffer. We do not use this data in any way. */
            apiRetStatus = CyU3PUsbGetEP0Data(CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED,
                    glCommitCtrl, &readCount);
            if (apiRetStatus != CY_U3P_SUCCESS)
            {
                initErrors |= 1 << 26;
//                CyU3PDebugPrint(4, "CyU3PUsbGetEP0Data failed, Error code = 0x%x\n", apiRetStatus);
            }

            /* Check the read count. Expecting a count of CY_FX_UVC_MAX_PROBE_SETTING bytes. */
            if (readCount != (uint16_t)CY_FX_UVC_MAX_PROBE_SETTING)
            {
                initErrors |= 1 << 27;
//                CyU3PDebugPrint(4, "Invalid number of bytes received in SET_CUR Request");
            }

            if (wValue == CY_FX_USB_UVC_VS_COMMIT_CONTROL)
            {
                /* We can start streaming video now. */
                apiRetStatus = CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_STREAM_EVENT, CYU3P_EVENT_OR);
                if (apiRetStatus != CY_U3P_SUCCESS)
                {
                    initErrors |= 1 << 28;
                    CyU3PDebugPrint (4, "Set CY_FX_UVC_STREAM_EVENT failed %x\r\n", apiRetStatus);
                }
            }
            break;

        default:
            CyU3PUsbStall(0, CyTrue, CyFalse);
            break;
        }
    }
}

/* Callback to handle the USB Setup Requests and UVC Class events */
static CyBool_t
CyFxUVCApplnUSBSetupCB (
        uint32_t setupdat0, /* SETUP Data 0 */
        uint32_t setupdat1  /* SETUP Data 1 */
        )
{
    CyBool_t isHandled = CyFalse;

    /* Decode the fields from the setup request. */
    bReqType =  (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);                       /* Direction, type and recipient of request */
    bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS); /* Specific request */
    wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);   /* Any value associated with request */
    wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);   /* Any index associated with request / addressed device */
    wLength  = ((setupdat1 & CY_U3P_USB_LENGTH_MASK)  >> CY_U3P_USB_LENGTH_POS);  /* Any index associated with request / addressed device */

    bType    =  (bReqType  & CY_U3P_USB_TYPE_MASK);   // Type of request
    bTarget  =  (bReqType  & CY_U3P_USB_TARGET_MASK); // Recipient of request

#if 0
    CyU3PDebugPrint(4, "UVC bReqType: 0x%x\r\n", bReqType);
    CyU3PDebugPrint(4, "UVC bType:    0x%x\r\n", bType);
    CyU3PDebugPrint(4, "UVC bTarget:  0x%x\r\n", bTarget);
    CyU3PDebugPrint(4, "UVC bRequest: 0x%x\r\n", bRequest);
    CyU3PDebugPrint(4, "UVC wValue:   0x%x\r\n", wValue);
    CyU3PDebugPrint(4, "UVC wIndex:   0x%x\r\n", wIndex);
#endif

    /* Check for UVC Class Requests */
    switch (bReqType)
    {
    case CY_FX_USB_UVC_GET_REQ_TYPE: /* 0b10100001 */
    case CY_FX_USB_UVC_SET_REQ_TYPE: /* 0b00100001 */
        /* UVC specific requests to UVC entities */
        switch (wIndex & 0xFF)
        {
        case CY_FX_UVC_INTERFACE_VC: /* Request directed to Video Control interface */
            {
                isHandled = CyTrue;
                CyFxUVCHandleControlRequest();
            }
            break;

        case CY_FX_UVC_INTERFACE_VS: /* Request directed to VideoStreaming interface */
            {
                isHandled = CyTrue;
                CyFxUVCHandleStreamingRequest();
            }
            break;

        }
        break;

    case CY_FX_USB_SET_INTF_REQ_TYPE: /* 0x01 */ /* Request directed to Interface */
        if (bRequest == CY_FX_USB_SET_INTERFACE_REQ)
        {
            /* Some hosts send Set Interface Alternate Setting 0 command while stopping the video
             * stream. The application uses this event to stop streaming. */
            if ((wIndex == CY_FX_UVC_INTERFACE_VS) && (wValue == 0))
            {
                /* Stop GPIF state machine to stop data transfers through FX3 */
                CyU3PDebugPrint (4, "Alternate setting 0..\r\n");

                /* Clear the stall condition and sequence numbers. */
                CyU3PUsbStall (CY_FX_EP_BULK_VIDEO, CyFalse, CyTrue);
                CyFxUVCApplnAbortHandler ();

                isHandled = CyTrue;
                /* Complete Control request handshake */
                CyU3PUsbAckSetup ();
            }
        }
        else if ((bRequest == CY_U3P_USB_SC_SET_FEATURE) || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE))
        {
            CyU3PDebugPrint(4, "USBSetupCB:In SET_FTR %d::%d\r\n", glIsUVCApplnActive, glIsUVCDevConfigured);
            if (glIsUVCDevConfigured)
            {
                isHandled = CyTrue;
                CyU3PUsbAckSetup ();
            }
        }

        break;

    case CY_U3P_USB_TARGET_ENDPT: /* 0x02 */
        if (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)
        {
            if (wIndex == CY_FX_EP_BULK_VIDEO)
            {
                /* Windows OS sends Clear Feature Request after it stops streaming,
                 * however MAC OS sends clear feature request right after it sends a
                 * Commit -> SET_CUR request. Hence, stop the video streaming and clear
                 * the stall condition and sequence numbers */
                CyU3PDebugPrint (4, "Clear feature request detected...\r\n");

                /* Clear the stall condition and sequence numbers. */
                CyU3PUsbStall (CY_FX_EP_BULK_VIDEO, CyFalse, CyTrue);
                CyFxUVCApplnAbortHandler();

                isHandled = CyTrue;
                /* Complete Control request handshake */
                CyU3PUsbAckSetup ();
            }
        }
        break;

    default:
        break;
    }

    return isHandled;
}

/* Entry function for the UVC application thread. */
void
UVCAppThread_Entry (
        uint32_t input)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint32_t            eventsOfInterest;
    uint32_t            flag;

    eventsOfInterest = CY_FX_UVC_STREAM_ABORT_EVENT | /* Host requests a stream to stop */
                       CY_FX_UVC_STREAM_EVENT |       /* Host requests a stream to start */
                       CY_FX_UVC_DMA_RESET_EVENT |    /* DMA requested a restart for some reason */
                       CY_FX_UVC_DMA_CB;

    for (;;)
    {
        apiRetStatus = CyU3PEventGet (&glFxUVCEvent, eventsOfInterest , CYU3P_EVENT_OR_CLEAR, &flag, LOOP_TIMEOUT);

        if (apiRetStatus == CY_U3P_SUCCESS)
        {
            if ((flag & CY_FX_UVC_STREAM_EVENT) != 0)
            {
                CyU3PDebugPrint(4, "UVC Event: Stream Start\r\n");

                glIsUVCApplnActive = CyTrue;
                CyFxUVCApplnStart();
            }

            if ((flag & CY_FX_UVC_STREAM_ABORT_EVENT) != 0)
            {
                CyU3PDebugPrint(4, "UVC Event: Stream Abort\r\n");

                glIsUVCApplnActive = CyFalse;
                CyFxUVCApplnStop();
            }

            if ((flag & CY_FX_UVC_DMA_CB) != 0)
            {
                CyU3PDebugPrint(4, "UVC Event: DMA Callback\r\n");
            }

            if (((flag & CY_FX_UVC_DMA_RESET_EVENT) != 0) && glIsUVCApplnActive)
            {
                if(glDmaResetFlag == CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE)
                    CyU3PDebugPrint (4, "DMA Reset Event: Commit buffer failure\r\n");

                CyFxUVCApplnStop();

                if ((glIsUVCApplnActive) && (++glCommitBufferFailureCount < CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT))
                {
                    CyFxUVCApplnStart(); /* Resets glDmaResetFlag */
                }

                /* Reset the video streaming flags for a MAC OS */
                if(glCommitBufferFailureCount == CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT)
                {
                    glIsUVCApplnActive = CyFalse;
                    CyU3PDebugPrint (4, "Application Stopped after %d Commit buffer failures\r\n", glCommitBufferFailureCount);
                    glCommitBufferFailureCount = 0;
                }
            }
        }
    }
}

/* Entry function for the UVC application thread. */
void
UVCAppEp0Thread_Entry (
        uint32_t input)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint8_t gpifState, dmaState;
    uint32_t prodCount, consCount;
    uint8_t outpin = CyFalse;

    for (;;)
    {
        /* Update current state variables. See if system is locking up due to error or resets */
        CyU3PGpifGetSMState(&gpifState);
        CyU3PDmaChannelGetStatus(&glChHandleUVCStream, &dmaState, &prodCount, &consCount);

        /* Display application statistics */
        CyU3PDebugPrint(4, "App Status\r\n");
        CyU3PDebugPrint(4, " Config/Active/Steaming: %d/%d/%d\r\n", glIsUVCDevConfigured,
                glIsUVCApplnActive, glStreamingStarted);
        CyU3PDebugPrint(4, " GPIF Throughput %d++/%d--\r\n", glEventProd, glEventCons); /* Should be same as following */
        CyU3PDebugPrint(4, " DMA State: %s (%d++/%d--)\r\n", getDmaStateString(dmaState),
                prodCount, consCount);
        CyU3PDebugPrint(4, " SM State: %s, %d CBs\r\n", getGpifStateString(gpifState), glEnterCallback);
        CyU3PDebugPrint(4, "  IDLE:      %d\t\tDSS_STATE: %d\t\tREAD:      %d\r\n",
                glEnterIdle, glEnterDss_state, glEnterRead);
        CyU3PDebugPrint(4, "  WRITE:     %d\t\tSHORT_PKT: %d\t\tZLP:       %d\r\n",
                glEnterWrite, glEnterShort_pkt, glEnterZlp);
        CyU3PDebugPrint(4, " GPIO State: %d\r\n", outpin);
        if (initErrors != 0)
                CyU3PDebugPrint(4, " Init Errors: 0x%x\r\n", initErrors);

        /* Toggle GPIO 22 value */
        apiRetStatus = CyU3PGpioSetValue (TIMING_PULSE_GPIO, outpin);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            /* Error handling */
            CyU3PDebugPrint (4, "CyU3PGpioSetValue failed, error code = %d\n", apiRetStatus);
            CyFxAppErrorHandler(apiRetStatus);
        }
        outpin = !outpin;

        CyU3PThreadSleep(5000);  /* Prevent continuous printing */
        CyU3PThreadRelinquish(); /* Allow other ready threads to run. */
    }
}

/* Entry function for the UVC application thread. */
void
UVCAppProdThread_Entry (
        uint32_t input)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    static uint8_t  flag = 0; /* Track the buffer currently being transmitted to host */
    int frame_cnt = 0;

    mod = MOD; /* Number of full frames to send : 253 full buffers */
    rem = REM; /* Byte length of last frame : 5084 bytes */

    CyU3PDebugPrint(4, "mod = %d\r\n", mod);
    CyU3PDebugPrint(4, "rem = %d\r\n", rem);

/*
    i = 0;
    CyU3PDmaState_t state;
    uint32_t p_count,c_count;
    status = CyU3PDmaChannelGetStatus(&glChHandleUVCStream, &state, &p_count, &c_count);
    if(status != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "1 UVC Failed to CyU3PDmaChannelGetStatus, Error code = 0x%x\r\n", status);
    }
    CyU3PDebugPrint (4, "1 state = %d, p_count = %d, c_count = %d\r\n", state, p_count, c_count);
*/

    for (;;)
    {
        /* Reset Frame Id in UVC Header */
        glUVCHeader[1] = CY_FX_UVC_HEADER_DEFAULT_BFH;

        /* Video streamer application. */
        while (glIsUVCApplnActive)
        {
            /* If there are full frames left to send */
            if(mod > 0)
            {
                /* Add standard frame header to buffer */
                CyFxUVCAddHeader(UvcdmaBuffer[flag].buffer, CY_FX_UVC_HEADER_FRAME);

                /* Update buffer metadata */
                UvcdmaBuffer[flag].count  = CY_FX_UVC_STREAM_BUF_SIZE; /* Size of valid data in buffer in bytes */
                UvcdmaBuffer[flag].size   = CY_FX_UVC_STREAM_BUF_SIZE; /* Maximum buffer size in bytes */
                UvcdmaBuffer[flag].status = 0;

                /* Send buffer to consumer (USB) through DMA channel */
                apiRetStatus = CyU3PDmaChannelSetupSendBuffer(&glChHandleUVCStream, &UvcdmaBuffer[flag]);
                if (apiRetStatus != CY_U3P_SUCCESS)
                {
                    CyU3PDebugPrint(4, "UVC Failed to CyU3PDmaChannelSetupSendBuffer_0, Error code = 0x%x\r\n", apiRetStatus);
                    break;
                }

                /* Wait for the current DMA transaction to complete */
                apiRetStatus = CyU3PDmaChannelWaitForCompletion(&glChHandleUVCStream, CYU3P_WAIT_FOREVER);
                if(apiRetStatus != CY_U3P_SUCCESS)
                {
                    CyU3PDebugPrint(4, "UVC Failed to CyU3PDmaChannelWaitForCompletion_0, Error code = 0x%x\r\n", apiRetStatus);
                    break;
                }

                /* Every 32 buffers wait... why? */
                if((mod & 0x1f) == 0x00)
                {
                    CyU3PThreadSleep(1);
                }

                /* Decrement number of full buffers to send */
                mod--;
            }
            else
            {
                /* Add End-of-frame header to buffer */
                CyFxUVCAddHeader(UvcdmaBuffer[flag].buffer, CY_FX_UVC_HEADER_EOF);

                //CyU3PDebugPrint (4, "UvcdmaBuffer[%d].buffer[1] = 0x%x\r\n", flag, UvcdmaBuffer[flag].buffer[1]);

                /* Update buffer metadata */
                UvcdmaBuffer[flag].count = rem + CY_FX_UVC_MAX_HEADER;
                UvcdmaBuffer[flag].size = CY_FX_UVC_STREAM_BUF_SIZE;
                UvcdmaBuffer[flag].status = 0;

                /* Send buffer to consumer (USB) */
                apiRetStatus = CyU3PDmaChannelSetupSendBuffer(&glChHandleUVCStream, &UvcdmaBuffer[flag]);
                if (apiRetStatus != CY_U3P_SUCCESS)
                {
                    CyU3PDebugPrint(4, "Failed to CyU3PDmaChannelSetupSendBuffer_1, Error code = 0x%x\r\n", apiRetStatus);
                    break;
                }

                /*Wait for the current DMA transaction to complete*/
                apiRetStatus = CyU3PDmaChannelWaitForCompletion(&glChHandleUVCStream, CYU3P_WAIT_FOREVER);
                if(apiRetStatus != CY_U3P_SUCCESS)
                {
                    CyU3PDebugPrint(4, "UVC Failed to CyU3PDmaChannelWaitForCompletion_1, Error code = 0x%x\r\n", apiRetStatus);
                    continue;
                }

                /* Toggle the index for the buffer being used. Effectively switches between the
                 * 2 randomly generated frames in CyFxApplicationDefine */
                flag = (flag + 1) % CY_FX_UVC_MAX_VID_FRAMES; /* increment in a way to go up to max frames */
                /* Reset number of full buffers to send (b/c we are now sending the other frame */
                mod = MOD;
                /* Increment the number of sent frames */
                frame_cnt++;

//              CyU3PDebugPrint (4, "send one frame over, flag = %d,frame_cnt = %d\r\n", flag, frame_cnt);
                CyU3PThreadSleep (1);
            }

            /* Move the USB link to U0 if we are stuck in U1/U2. */
            if (CyU3PUsbGetSpeed() == CY_U3P_SUPER_SPEED)
            {
                CyU3PUsbLinkPowerMode u3mode;

                if ((CyU3PUsbGetLinkPowerState(&u3mode) == CY_U3P_SUCCESS) && (
                            (u3mode == CyU3PUsbLPM_U1) || (u3mode == CyU3PUsbLPM_U2)))
                {
                    CyU3PUsbSetLinkPowerState(CyU3PUsbLPM_U0);
                }
            }
        }

        /* There is a streamer error. Flag it. */
        if ((apiRetStatus != CY_U3P_SUCCESS) && (glIsUVCApplnActive))
        {
            CyU3PDebugPrint(4, "Main Loop: UVC video streamer error. Error code 0x%x.\r\n", apiRetStatus);
            CyFxAppErrorHandler (apiRetStatus);
        }

        /* Sleep for sometime as video streamer is idle. */
        CyU3PThreadSleep (100);
    } /* End of for(;;) */
}

#endif /* USB_UVC_CLASS */


#if USB_UAC_CLASS == 1

void
CyFxUACApplnDmaCallback (
        CyU3PDmaChannel *chHandle,
        CyU3PDmaCbType_t type,
        CyU3PDmaCBInput_t *input
        )
{
    if (type == CY_U3P_DMA_CB_CONS_EVENT || type == CY_U3P_DMA_CB_SEND_CPLT)
    {
        CyU3PDebugPrint (4, "CyFxUACApplnDmaCallback\r\n");
    }
}

/* This function starts the audio streaming application. It is called
 * when there is a SET_INTERFACE event for alternate interface 1. */
CyU3PReturnStatus_t
CyFxUACApplnStart (void)
{
    CyU3PEpConfig_t epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Audio streaming endpoint configuration */
    epCfg.enable = CyTrue;
    epCfg.epType = CY_U3P_USB_EP_ISO;
    epCfg.pcktSize = CY_FX3_ISO_XFER_LEN;
    epCfg.isoPkts = 1;
    epCfg.burstLen = 1;
    epCfg.streams = 0;

    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_ISO_AUDIO, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = 0x%x\n", apiRetStatus);
        return apiRetStatus;
    }

    /* Create a DMA Manual OUT channel for streaming data */
    /* Audio streaming Channel is not active till a stream request is received */
    dmaCfg.size = CY_FX3_ISO_XFER_LEN;
    //dmaCfg.count = CY_FX_UAC_STREAM_BUF_COUNT;
    dmaCfg.count = 0;                   /*add,No buffers allocated. We will only use the SetupSend API*/
    dmaCfg.prodSckId = CY_U3P_CPU_SOCKET_PROD;
    dmaCfg.consSckId = CY_FX_EP_AUDIO_CONS_SOCKET;
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    //dmaCfg.cb = CyFxUACApplnDmaCallback;
    //dmaCfg.notification = CY_U3P_DMA_CB_CONS_EVENT | CY_U3P_DMA_CB_SEND_CPLT;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;
    apiRetStatus = CyU3PDmaChannelCreate (&glUacStreamHandle, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "UAC CyU3PDmaChannelCreate failed, Error code = 0x%x\n",apiRetStatus);
        return apiRetStatus;
    }

    if (dmaCfg.count != 0)
    {
        /* Flush the endpoint memory */
        CyU3PUsbFlushEp (CY_FX_EP_ISO_AUDIO);

        /*add,Setup a one-to-one DMA channel for data transfer*/
        apiRetStatus = CyU3PDmaChannelSetXfer (&glUacStreamHandle, 0);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed, Error code = 0x%x\n", apiRetStatus);
            return apiRetStatus;
        }
    }

    /* Update the flag so that the application thread is notified of this. */
    glIsUACApplnActive = CyTrue;
    CyU3PDebugPrint (4, "UAC App Started\r\n");
    return CY_U3P_SUCCESS;
}

/* This function stops the audio streaming. It is called from the USB event
 * handler, when there is a reset / disconnect or SET_INTERFACE for alternate
 * interface 0. */
void
CyFxUACApplnStop (void)
{
    CyU3PEpConfig_t epCfg;

    /* Update the flag so that the application thread is notified of this. */
    glIsUACApplnActive = CyFalse;

    /* Abort and destroy the Audio streaming channel */
    CyU3PDmaChannelDestroy (&glUacStreamHandle);

    /* Flush the endpoint memory */
    //CyU3PUsbFlushEp (CY_FX_EP_ISO_AUDIO);

    /* Disable the audio streaming endpoint. */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyFalse;

    CyU3PSetEpConfig (CY_FX_EP_ISO_AUDIO, &epCfg);
    CyU3PDebugPrint (4, "UAC App Stopped\r\n");
}

/* This is the Callback function to handle the USB Events */
static void
CyFxUACApplnUsbEventCB (
    CyU3PUsbEventType_t evtype, /* Event type */
    uint16_t            evdata  /* Event data */
    )
{
//  CyU3PDebugPrint (4, "UAC CyFxUACApplnUsbEventCB\r\n");

    uint8_t interface = 0, altSetting = 0;

#if 0
    CyU3PDebugPrint (4, "UAC evtype: 0x%x\r\n", evtype);
    CyU3PDebugPrint (4, "UAC evdata: 0x%x\r\n", evdata);
#endif

    switch (evtype)
    {
    case CY_U3P_USB_EVENT_SETCONF:                                      /*add 20210824,CY_U3P_USB_EVENT_SETCONF=5*/
        /* Disable the low power entry to optimize USB throughput */
        CyU3PUsbLPMDisable();
        /* Stop application on SET_CONFIG. It will be started on SET_INTF. */
        if (glIsUACApplnActive)
            CyFxUACApplnStop ();
        if (evdata != 0)
            glIsDevConfigured = CyTrue;
        else
            glIsDevConfigured = CyFalse;
        break;

    case CY_U3P_USB_EVENT_SETINTF:                                      /*add 20210824,CY_U3P_USB_EVENT_SETINTF=7*/
         /* Start the audio streamer application if the
          * interface requested was 1. If not, stop the
          * streamer. */
        interface = CY_U3P_GET_MSB(evdata);
        altSetting = CY_U3P_GET_LSB(evdata);

        if ((altSetting == CY_FX_UAC_STREAM_INTERFACE) &&
                (interface == CY_FX_UAC_INTERFACE_VS))
        {
            /* Stop the application before re-starting. */
            if (glIsUACApplnActive)
            {
                CyFxUACApplnStop ();
            }
            CyFxUACApplnStart ();
            break;
        }
        break;
        /* Fall-through. */

    case CY_U3P_USB_EVENT_RESET:                                    /*add 20210824,CY_U3P_USB_EVENT_RESET=4,CY_U3P_USB_EVENT_DISCONNECT=1*/
    case CY_U3P_USB_EVENT_DISCONNECT:
        /* Stop the audio streamer application. */
        if (glIsUACApplnActive)
        {
            CyFxUACApplnStop ();
        }
        break;

    default:
        break;
    }
}

/* Callback to handle the USB Setup Requests and UAC Class requests */
static CyBool_t
CyFxUACApplnUSBSetupCB (
        uint32_t setupdat0, /* SETUP Data 0 */
        uint32_t setupdat1  /* SETUP Data 1 */
    )
{
    CyU3PDebugPrint (4, "UAC CyFxUACApplnUSBSetupCB\r\n");

    uint8_t  bRequest, bReqType;
    uint8_t  bType, bTarget;
    uint16_t wValue,wIndex;
    CyBool_t isHandled = CyFalse;

    /* Fast enumeration is used. Only requests addressed to the interface, class,
     * vendor and unknown control requests are received by this function.
     * */

    /* Decode the fields from the setup request. */
    bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);                                      /*CY_U3P_USB_REQUEST_TYPE_MASK=0xff*/
    bType    = (bReqType & CY_U3P_USB_TYPE_MASK);                                               /*CY_U3P_USB_TYPE_MASK=0x60*/
    bTarget  = (bReqType & CY_U3P_USB_TARGET_MASK);                                             /*CY_U3P_USB_TARGET_MASK=0x03*/
    bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
    wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);                 /*CY_U3P_USB_VALUE_MASK=0xffff0000,CY_U3P_USB_VALUE_POS=16*/
    wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);                 /*CY_U3P_USB_INDEX_MASK=0x0000ffff,CY_U3P_USB_INDEX_POS=0*/

#if 0
    CyU3PDebugPrint (4, "UAC bReqType: 0x%x\r\n", bReqType);
    CyU3PDebugPrint (4, "UAC bType: 0x%x\r\n", bType);
    CyU3PDebugPrint (4, "UAC bTarget: 0x%x\r\n", bTarget);
    CyU3PDebugPrint (4, "UAC bRequest: 0x%x\r\n", bRequest);
    CyU3PDebugPrint (4, "UAC wValue: 0x%x\r\n", wValue);
    CyU3PDebugPrint (4, "UAC wIndex: 0x%x\r\n", wIndex);
#endif

    if (bType == CY_U3P_USB_STANDARD_RQT)                                   /*add 20210824,CY_U3P_USB_STANDARD_RQT=0x00*/
    {
        /* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
         * requests here. It should be allowed to pass if the device is in configured
         * state and failed otherwise. */
        if ((bTarget == CY_U3P_USB_TARGET_INTF) && ((bRequest == CY_U3P_USB_SC_SET_FEATURE)     /*CY_U3P_USB_TARGET_INTF=0x01,CY_U3P_USB_SC_SET_FEATURE=3,CY_U3P_USB_SC_CLEAR_FEATURE=1*/
                    || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) && (wValue == 0))
        {
            glIsDevConfigured = CyTrue;
            if (glIsDevConfigured)
                CyU3PUsbAckSetup();
            else
                CyU3PUsbStall(0, CyTrue, CyFalse);

            isHandled = CyTrue;
        }
    }

    /* Check for UAC Class Requests */
    if (bType == CY_U3P_USB_CLASS_RQT)                                      /*add 20210824,CY_U3P_USB_CLASS_RQT=0x20*/
    {
        while (bType == bType)
        {
            CyU3PBusyWait(10);
        }
    }

    return isHandled;
}

/* Callback function to handle LPM requests from the USB 3.0 host. This function is invoked by the API
 * whenever a state change from U0 -> U1 or U0 -> U2 happens. If we return CyTrue from this function,
 * the FX3 device is retained in the low power state. If we return CyFalse, the FX3 device immediately
 * tries to trigger an exit back to U0.
 * This application does not have any state in which we should not allow U1/U2 transitions; and therefore
 * the function always return CyTrue.
 * */
CyBool_t
CyFxUACApplnLPMRqtCB (
        CyU3PUsbLinkPowerMode link_mode)
{
    return CyTrue;
}

/* Entry function for the UAC application thread. */
void
UACAppThread_Entry (
        uint32_t input)
{
    //uint16_t offset = 0;
    uint16_t dataLen = 96*2;
    CyU3PDebugPrint (4, "dataLen = %d\r\n", dataLen);

    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    uint8_t flag = 0;

    for (;;)
    {
#if 1
        /* Audio streaming logic */
        while (glIsUACApplnActive)
        {
            status = CyU3PDmaChannelSetupSendBuffer (&glUacStreamHandle, &UACDmaBuffer[flag]);
            if (status != CY_U3P_SUCCESS)
            {
                CyU3PDebugPrint (4, "Failed to CyU3PDmaChannelSetupSendBuffer, Error code = 0x%x\r\n", status);
                break;
            }

            /*Wait for the current DMA transaction to complete*/
            status = CyU3PDmaChannelWaitForCompletion (&glUacStreamHandle, CYU3P_WAIT_FOREVER);
            //if(status != CY_U3P_SUCCESS)
            {
                //CyU3PDebugPrint (4, "UAC Failed to CyU3PDmaChannelWaitForCompletion, Error code = 0x%x\r\n", status);
                //break;
            }

            if(flag == 1)
            {
                CyU3PThreadSleep (1);
            }

            flag ^= 1;
        }

        /* There is a streamer error. Flag it. */
        if ((status != CY_U3P_SUCCESS) && (glIsUACApplnActive))
        {
            CyU3PDebugPrint (4, "UAC streamer error. Error code 0x%x.\n", status);
            CyFxAppErrorHandler (status);
        }
#endif
        /* Sleep for sometime as audio streamer is idle. */
        CyU3PThreadSleep (100);
    } /* End of for(;;) */
}

#endif /* USB_UAC_CLASS */


#if USB_CDC_CLASS == 1

/* USBUart receive data callback */
void
CyFxUARTApplnDmaRecvCallback (
        CyU3PDmaChannel *chHandle,
        CyU3PDmaCbType_t type,
        CyU3PDmaCBInput_t *input
        )
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PDmaBuffer_t    dmaBuffer;
    uint8_t i = 0;

    CyU3PDebugPrint (4, "CyFxUARTApplnDmaRecvCallback\r\n");

    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
        status = CyU3PDmaChannelGetBuffer (chHandle, &dmaBuffer, CYU3P_NO_WAIT);        /*add,Get the current buffer pointer*/
        while (status == CY_U3P_SUCCESS)
        {
            /*one frame*/
            if (dmaBuffer.count == FRAME_SIZE)          /*add 64byte,Byte count of valid data in buffer*/
            {
                /*frame parsing*/
                for(i = 0;i < FRAME_SIZE;i++)
                {
                    CyU3PDebugPrint (4, "dmaBuffer.buffer[%d] = 0x%x\r\n",i, dmaBuffer.buffer[i]);
                }

                for(i = 0;i < FRAME_SIZE;i++)
                {
                    USB2UARTResp[i] = i;
                }

                CyU3PMemCopy (dmaBuffer.buffer,(uint8_t *)USB2UARTResp,FRAME_SIZE);
                dmaBuffer.count = FRAME_SIZE;
                dmaBuffer.size = FRAME_SIZE;
                dmaBuffer.status = 0;

                /* Wait until the response has gone out. */
                status = CyU3PDmaChannelWaitForCompletion (&glChHandleUsb2UartTrans, CYU3P_WAIT_FOREVER);

                status = CyU3PDmaChannelSetupSendBuffer (&glChHandleUsb2UartTrans, &dmaBuffer);
                if (status != CY_U3P_SUCCESS)
                {
                    CyU3PDebugPrint (4, "Failed to CyU3PDmaChannelSetupSendBuffer, Error code = 0x%x\r\n", status);
                    return;
                }
            }
            else    /*Lack of a frame*/
            {
                /* A partially filled buffer indicates the end of the ongoing video frame. */
                return;
            }
        }
    }
}

/* USBUart transmit data callback */
void
CyFxUARTApplnDmaTransCallback(
        CyU3PDmaChannel   *chHandle, /* Handle to the DMA channel. */
        CyU3PDmaCbType_t   type,     /* Callback type.             */
        CyU3PDmaCBInput_t *input)    /* Callback status.           */
{
    if (type == CY_U3P_DMA_CB_CONS_EVENT)
    {
        CyU3PDebugPrint (4, "USB2UART send over\r\n");
    }
}

/* This function starts the USBUART application */
void
CyFxUSBUARTAppStart(void)
{
    uint16_t size = 0;
    CyU3PEpConfig_t epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

    /* Based on the Bus speed configure the endpoint packet size */
    switch (usbSpeed)
    {
        case CY_U3P_FULL_SPEED:
            size = 64;
            break;

        case CY_U3P_HIGH_SPEED:
            size = 512;
            break;

        case  CY_U3P_SUPER_SPEED:
            /* Turning low power mode off to avoid USB transfer delays. */
            CyU3PUsbLPMDisable ();
            size = 1024;
            break;

        default:
            CyFxAppErrorHandler (CY_U3P_ERROR_FAILURE);
            break;
    }

    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyTrue;
    epCfg.epType = CY_U3P_USB_EP_BULK;
    epCfg.burstLen = 1;
    epCfg.streams = 0;
    epCfg.pcktSize = size;

    /* Producer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER , &epCfg);    /*add,OUT ENDPOINT,uart receive data endpoint of 0x01*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Consumer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);     /*add,IN ENDPOINT,uart transmit data endpoint of 0x85*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Interrupt endpoint configuration */
    epCfg.epType = CY_U3P_USB_EP_INTR;
    epCfg.pcktSize = 64;
    epCfg.isoPkts = 1;

    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_INTERRUPT, &epCfg);    /*add,IN ENDPOINT,uart transmit cmd endpoint of 0x84*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Create a DMA_MANUAL channel between usb2uart producer socket and cpu consumer socket */
    dmaCfg.size = FRAME_SIZE;
    dmaCfg.count = CY_FX_USBUART_DMA_BUF_COUNT;         /*0x1*/
    dmaCfg.prodSckId = CY_U3P_UIB_SOCKET_PROD_5;        /*0x0405*/
    dmaCfg.consSckId = CY_U3P_CPU_SOCKET_CONS;          /*0x3F00*/
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT;     /*finish receive data trigger interrupt*/
    dmaCfg.cb = CyFxUARTApplnDmaRecvCallback;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;

    apiRetStatus = CyU3PDmaChannelCreate (&glChHandleUsb2UartRecv, CY_U3P_DMA_TYPE_MANUAL_IN, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Create a DMA_MANUAL channel between cpu producer socket and usb2uart consumer socket */
    /* Use a smaller buffer size (32 bytes) to ensure that packets get filled in a short time. */
    dmaCfg.size         = FRAME_SIZE;
    //dmaCfg.count      = CY_FX_USBUART_DMA_BUF_COUNT;          /*0x1*/
    dmaCfg.count        = 0;
    dmaCfg.prodSckId    = CY_U3P_CPU_SOCKET_PROD;           /*0x3F01*/
    dmaCfg.consSckId    = CY_U3P_UIB_SOCKET_CONS_5;         /*0x0305*/
    dmaCfg.notification = CY_U3P_DMA_CB_CONS_EVENT | CY_U3P_DMA_CB_SEND_CPLT;       /*finish transmit data trigger interrupt*/
    dmaCfg.cb           = CyFxUARTApplnDmaTransCallback;

    apiRetStatus = CyU3PDmaChannelCreate (&glChHandleUsb2UartTrans, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp (CY_FX_EP_PRODUCER);

    /* Flush the endpoint memory */
    //CyU3PUsbFlushEp (CY_FX_EP_CONSUMER);

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_INTERRUPT);

    /* Set DMA Channel transfer size */
    apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleUsb2UartRecv,0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    //apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleUsb2UartTrans, 0);
    //if (apiRetStatus != CY_U3P_SUCCESS)
    {
        //CyFxAppErrorHandler(apiRetStatus);
    }

    USB2UARTResp = CyU3PDmaBufferAlloc(FRAME_SIZE);
    if(USB2UARTResp == NULL)
    {
        CyU3PDebugPrint (4, "USB2UARTResp == NULL\r\n");
        return;
    }

    CyU3PDebugPrint (4, "CDC App Started\r\n");

    /* Update the status flag. */
    glIsApplnActive = CyTrue;
}

/* This function stops the USBUART application */
void
CyFxUSBUARTAppStop (void)
{
    CyU3PEpConfig_t epCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Update the flag. */
    glIsApplnActive = CyFalse;

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
    //CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);
    CyU3PUsbFlushEp(CY_FX_EP_INTERRUPT);

    /* Destroy the channel */
    CyU3PDmaChannelDestroy (&glChHandleUsb2UartRecv);
    CyU3PDmaChannelDestroy (&glChHandleUsb2UartTrans);

    /* Disable endpoints. */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyFalse;

    /* Producer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Consumer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Interrupt endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_INTERRUPT, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }
}

/* This is the callback function to handle the USB events. */
void
CyFxUSBUARTAppUSBEventCB (
        CyU3PUsbEventType_t evtype, /* Event type */
        uint16_t            evdata  /* Event data */
        )
{

#if 0
    CyU3PDebugPrint (4, "CDC evtype: 0x%x\r\n", evtype);
    CyU3PDebugPrint (4, "CDC evdata: 0x%x\r\n", evdata);
#endif

    switch (evtype)
    {
    case CY_U3P_USB_EVENT_SETCONF:
        /* Stop the application before re-starting. */
        if (glIsApplnActive)
        {
            CyFxUSBUARTAppStop ();
        }
        /* Start the loop back function. */
        CyFxUSBUARTAppStart ();
        break;

    case CY_U3P_USB_EVENT_RESET:
    case CY_U3P_USB_EVENT_CONNECT:
    case CY_U3P_USB_EVENT_DISCONNECT:
        /* Stop the loop back function. */
        if (glIsApplnActive)
        {
            CyU3PUsbLPMEnable ();
            CyFxUSBUARTAppStop ();
        }
        break;

    default:
        break;
    }
}

/* Callback to handle the USB Setup Requests and CDC Class events */
static CyBool_t
CyFxUSBUARTAppUSBSetupCB (
        uint32_t setupdat0, /* SETUP Data 0 */
        uint32_t setupdat1  /* SETUP Data 1 */
        )
{
    uint16_t readCount = 0;
    uint8_t  bRequest, bReqType;
    uint8_t  bType, bTarget;
    uint16_t wValue,wIndex;
    uint8_t config_data[7];     /*add,host send data,host com configuration information*/
    CyBool_t isHandled = CyFalse;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PUartConfig_t uartConfig;

    /* Fast enumeration is used. Only requests addressed to the interface, class,
     * vendor and unknown control requests are received by this function. */

    /* Decode the fields from the setup request. */
    bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);                                      /*CY_U3P_USB_REQUEST_TYPE_MASK=0xff*/
    bType    = (bReqType & CY_U3P_USB_TYPE_MASK);                                               /*CY_U3P_USB_TYPE_MASK=0x60*/
    bTarget  = (bReqType & CY_U3P_USB_TARGET_MASK);                                             /*CY_U3P_USB_TARGET_MASK=0x03*/
    bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);               /*CY_U3P_USB_REQUEST_MASK=0xff00,CY_U3P_USB_REQUEST_POS=8*/
    wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);                 /*CY_U3P_USB_VALUE_MASK=0xffff0000,CY_U3P_USB_VALUE_POS=16*/
    wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);                 /*add 20210823,CY_U3P_USB_INDEX_MASK=0x0000FFFF,CY_U3P_USB_INDEX_POS=0*/

#if 0
    CyU3PDebugPrint (4, "CDC bReqType: 0x%x\r\n", bReqType);
    CyU3PDebugPrint (4, "CDC bType: 0x%x\r\n", bType);
    CyU3PDebugPrint (4, "CDC bTarget: 0x%x\r\n", bTarget);
    CyU3PDebugPrint (4, "CDC bRequest: 0x%x\r\n", bRequest);
    CyU3PDebugPrint (4, "CDC wValue: 0x%x\r\n", wValue);
    CyU3PDebugPrint (4, "CDC wIndex: 0x%x\r\n", wIndex);
#endif

    if (bType == CY_U3P_USB_STANDARD_RQT)
    {
        /* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
         * requests here. It should be allowed to pass if the device is in configured
         * state and failed otherwise. */
        if ((bTarget == CY_U3P_USB_TARGET_INTF) && ((bRequest == CY_U3P_USB_SC_SET_FEATURE)
                    || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) && (wValue == 0))
        {
            if (glIsApplnActive)
                CyU3PUsbAckSetup ();
            else
                CyU3PUsbStall (0, CyTrue, CyFalse);

            isHandled = CyTrue;
        }
    }

    /* Check for CDC Class Requests */
    if (bType == CY_U3P_USB_CLASS_RQT)
    {
        isHandled = CyTrue;

        /* CDC Specific Requests */
        /* set_line_coding */
        if (bRequest == SET_LINE_CODING)
        {
            CyU3PDebugPrint (4, "SET_LINE_CODING\r\n");

            status = CyU3PUsbGetEP0Data(0x07, config_data, &readCount);
            if (status != CY_U3P_SUCCESS)
            {
                CyFxAppErrorHandler(status);
            }
            if (readCount != 0x07)
            {
                CyFxAppErrorHandler(CY_U3P_ERROR_BAD_SIZE);
            }
            else
            {
                CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));

                uartConfig.baudRate = (CyU3PUartBaudrate_t)(config_data[0] | (config_data[1]<<8)|
                        (config_data[2]<<16)|(config_data[3]<<24));
                if (config_data[4] == 0)
                {
                    uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
                }
                else if (config_data[4] == 2)
                {
                    uartConfig.stopBit = CY_U3P_UART_TWO_STOP_BIT;
                }
                else
                {
                    /* Give invalid value. */
                    uartConfig.stopBit = (CyU3PUartStopBit_t)0;
                }
                if (config_data[5] == 1)
                {
                    uartConfig.parity = CY_U3P_UART_ODD_PARITY;
                }
                else if (config_data[5] == 2)
                {
                    uartConfig.parity = CY_U3P_UART_EVEN_PARITY;
                }
                else
                {
                    /* 0 = no parity; any other value - invalid parity. */
                    uartConfig.parity = CY_U3P_UART_NO_PARITY;
                }

                uartConfig.txEnable = CyTrue;
                uartConfig.rxEnable = CyTrue;
                uartConfig.flowCtrl = CyFalse;
                uartConfig.isDma = CyTrue;
#if 0
                /* Set the uart configuration */
                apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
                if (apiRetStatus == CY_U3P_SUCCESS)
                {
                    CyU3PMemCopy ((uint8_t *)&glUartConfig, (uint8_t *)&uartConfig,
                            sizeof (CyU3PUartConfig_t));
                }
#endif

                /*add*/
                CyU3PMemCopy ((uint8_t *)&glUartConfig, (uint8_t *)&uartConfig,
                                            sizeof (CyU3PUartConfig_t));
            }
        }
        /* get_line_coding */
        else if (bRequest == GET_LINE_CODING )
        {
            CyU3PDebugPrint (4, "GET_LINE_CODING\r\n");

            /* get current uart config */
            config_data[0] = glUartConfig.baudRate&(0x000000FF);
            config_data[1] = ((glUartConfig.baudRate&(0x0000FF00))>> 8);
            config_data[2] = ((glUartConfig.baudRate&(0x00FF0000))>>16);
            config_data[3] = ((glUartConfig.baudRate&(0xFF000000))>>24);

            if (glUartConfig.stopBit == CY_U3P_UART_ONE_STOP_BIT)
            {
                config_data[4] = 0;
            }
            else /* CY_U3P_UART_TWO_STOP_BIT */
            {
                config_data[4] = 2;
            }

            if (glUartConfig.parity == CY_U3P_UART_EVEN_PARITY)
            {
                config_data[5] = 2;
            }
            else if (glUartConfig.parity == CY_U3P_UART_ODD_PARITY)
            {
                config_data[5] = 1;
            }
            else
            {
                config_data[5] = 0;
            }
            config_data[6] =  0x08;
            status = CyU3PUsbSendEP0Data( 0x07, config_data);
            if (status != CY_U3P_SUCCESS)
            {
                CyFxAppErrorHandler(status);
            }
        }
        /* SET_CONTROL_LINE_STATE */
        else if (bRequest == SET_CONTROL_LINE_STATE)
        {
            CyU3PDebugPrint (4, "SET_CONTROL_LINE_STATE\r\n");

            if (glIsApplnActive)
            {
                CyU3PUsbAckSetup ();
            }
            else
                CyU3PUsbStall (0, CyTrue, CyFalse);
        }
        else
        {
            CyU3PDebugPrint (4, "others\r\n");
            status = CY_U3P_ERROR_FAILURE;
        }

        if (status != CY_U3P_SUCCESS)
        {
            isHandled = CyFalse;
        }
    }

    return isHandled;
}

/* Entry function for the USBUARTAppThread */
void
USBUARTAppThread_Entry (
        uint32_t input)
{
    CyU3PDebugPrint (4, "USBUARTAppThread_Entry start\r\n");
    for (;;)
    {
        CyU3PThreadSleep (100);
    }
}

#endif /* USB_CDC_CLASS */


/********************************************************************************
 *               Main Functions                                                 *
 ********************************************************************************/

/* Callback function to handle LPM requests from the USB 3.0 host. This function
 * is invoked by the API whenever a state change from U0 -> U1 or U0 -> U2 happens.
 * If we return CyTrue from this function, the FX3 device is retained in the low
 * power state. If we return CyFalse, the FX3 device immediately tries to trigger
 * an exit back to U0.
 *
 * This application does not have any state in which we should not allow U1/U2
 * transitions; and therefore the function always return CyTrue.
 *
 * For debugging we disable transitions by returning CyFalse
 */
CyBool_t
CyFxApplnLPMRqtCB (
        CyU3PUsbLinkPowerMode link_mode)
{
    return CyTrue;
}

/* This is the callback function to handle the USB events. */
static void
CyFxApplnUSBEventCB (
    CyU3PUsbEventType_t evtype, /* Event type */
    uint16_t            evdata  /* Event data */
    )
{
    uint8_t interface = CY_U3P_GET_MSB(evdata);

#if 0
    CyU3PDebugPrint (4, "Event for interface = %d (%s)\r\n", interface,
        interface == CY_FX_UVC_INTERFACE_VC  ? "UVC VC"   :
        interface == CY_FX_UVC_INTERFACE_VS  ? "UVC VS"   :
        interface == CY_FX_UAC_INTERFACE_VC  ? "UAC VC"   :
        interface == CY_FX_UAC_INTERFACE_VC  ? "UAC VC"   :
        interface == CY_FX_CDC_INTERFACE_CMD ? "CDC CMD"  :
                                               "CDC DATA"
    );

    CyU3PDebugPrint (4, " evtype: 0x%x\r\n", evtype);
    CyU3PDebugPrint (4, " evdata: 0x%x\r\n", evdata);
#endif

#if USB_UVC_CLASS == 1
    /*uvc event*/
    if(interface == CY_FX_UVC_INTERFACE_VC || interface == CY_FX_UVC_INTERFACE_VS)
    {
        CyFxUVCApplnUSBEventCB(evtype, evdata);
    }
#endif /* USB_UVC_CLASS */

#if USB_UAC_CLASS == 1
    /*uac event*/
    if(interface == CY_FX_UAC_INTERFACE_VC || interface == CY_FX_UAC_INTERFACE_VS)
    {
        CyFxUACApplnUsbEventCB(evtype, evdata);
    }
#endif /* USB_UAC_CLASS */

#if USB_CDC_CLASS == 1
    /*cdc event*/
    if(interface == CY_FX_CDC_INTERFACE_CMD || interface == CY_FX_CDC_INTERFACE_DATA)
    {
        CyFxUSBUARTAppUSBEventCB(evtype, evdata);
    }
#endif /* USB_CDC_CLASS */
}

/* Callback to handle the USB setup requests. */
static CyBool_t
CyFxApplnUSBSetupCB (
        uint32_t setupdat0,             /* SETUP Data 0 */      /*add 20210823,this parameter is struct of usb setup,setupdat0 is low 4 bytes*/
        uint32_t setupdat1              /* SETUP Data 1 */      /*add 20210823,setupdat1 is high 4 bytes*/
    )
{
    CyBool_t isHandled = CyFalse;
    uint16_t wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK) >> CY_U3P_USB_INDEX_POS);          /*add 20210824,CY_U3P_USB_INDEX_MASK=0x0000FFFF,CY_U3P_USB_INDEX_POS=0*/

#if USB_UVC_CLASS == 1
    /* UVC Setup */
    if((CY_U3P_GET_LSB(wIndex) == CY_FX_UVC_INTERFACE_VC) || (CY_U3P_GET_LSB(wIndex) == CY_FX_UVC_INTERFACE_VS))
    {
        isHandled = CyFxUVCApplnUSBSetupCB(setupdat0, setupdat1);
    }
#endif /* USB_UVC_CLASS */

#if USB_UAC_CLASS == 1
    /* UAC Setup*/
    if((CY_U3P_GET_LSB(wIndex) == CY_FX_UAC_INTERFACE_VC) || (CY_U3P_GET_LSB(wIndex) == CY_FX_UAC_INTERFACE_VS))
    {
        isHandled = CyFxUACApplnUSBSetupCB(setupdat0, setupdat1);
    }
#endif /* USB_UAC_CLASS */

#if USB_CDC_CLASS == 1
    /* CDC Setup */
    if((CY_U3P_GET_LSB(wIndex) == CY_FX_CDC_INTERFACE_CMD) || (CY_U3P_GET_LSB(wIndex) == CY_FX_CDC_INTERFACE_DATA))
    {
        isHandled = CyFxUSBUARTAppUSBSetupCB(setupdat0, setupdat1);
    }
#endif /* USB_CDC_CLASS */

    return isHandled;
}


/* This function initializes the debug module for the UVC application. */
void
CyFxApplnDebugInit (void)
{
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize the UART for printing debug messages */
    apiRetStatus = CyU3PUartInit();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        initErrors |= 1 << 0;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set UART Configuration */
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit  = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity   = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyFalse;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.isDma    = CyTrue;

    /* Set the UART configuration */
    apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        initErrors |= 1 << 1;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set the UART transfer */
    apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        initErrors |= 1 << 2;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Initialize the debug application */
    apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        initErrors |= 1 << 3;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Disable log message headers. */
    CyU3PDebugPreamble (CyFalse);

    CyU3PDebugPrint(8, "Logging initialized\r\n");
}

/* This function initializes the GPIO block, enabling GPIO[22] (G6) to be
 * used as an output for the timing pulse to the FPGA. */
void
CyFxApplnGpioInit(void)
{
    CyU3PGpioClock_t gpioClock;
    CyU3PGpioSimpleConfig_t gpioConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Init the GPIO module */
    gpioClock.fastClkDiv = 2;
    gpioClock.slowClkDiv = 0;
    gpioClock.simpleDiv = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
    gpioClock.clkSrc = CY_U3P_SYS_CLK;
    gpioClock.halfDiv = 0;
    apiRetStatus = CyU3PGpioInit(&gpioClock, NULL);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error Handling */
        CyU3PDebugPrint (4, "CyU3PGpioInit failed, error code = %d\n", apiRetStatus);
        initErrors |= 1 << 4;
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Unreserve GPIO 22 */
    apiRetStatus = CyU3PDeviceGpioOverride (
            TIMING_PULSE_GPIO, /* IO to be converted to GPIO */
            CyTrue             /* Simple GPIO */
    );
   if (apiRetStatus != CY_U3P_SUCCESS)
   {
       /* Error Handling */
       CyU3PDebugPrint (4, "CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
       initErrors |= 1 << 5;
       CyFxAppErrorHandler(apiRetStatus);
   }

   /* Configure GPIO 22 as an output */
   gpioConfig.outValue    = CyFalse; /* Initial out value */
   gpioConfig.inputEn     = CyFalse; /* Enable input stage */
   gpioConfig.driveLowEn  = CyTrue;  /* Output driver enabled for outValue = CyFalse */
   gpioConfig.driveHighEn = CyTrue;  /* Output driver enabled for outValue = CyTrue */
   gpioConfig.intrMode    = CY_U3P_GPIO_NO_INTR;
   apiRetStatus = CyU3PGpioSetSimpleConfig(TIMING_PULSE_GPIO, &gpioConfig);
   if (apiRetStatus != CY_U3P_SUCCESS)
   {
       /* Error handling */
       CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig failed, error code = %d\n", apiRetStatus);
       initErrors |= 1 << 6;
       CyFxAppErrorHandler(apiRetStatus);
   }
}

/* This function initializes the USB Module, creates event group, sets the
 * enumeration descriptors, configures the Endpoints and configures the DMA module
 * for the UVC Application
 */
void
CyFxApplnInit (void)
{
    CyU3PEpConfig_t         epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PPibClock_t         pibclock;
    CyU3PReturnStatus_t     apiRetStatus = CY_U3P_SUCCESS;

    /* Start the USB functionality. */
    apiRetStatus = CyU3PUsbStart();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Function Failed to Start, Error Code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 7;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Create event group for thread signaling */
    apiRetStatus = CyU3PEventCreate (&glFxUVCEvent);
    if (apiRetStatus != 0)
    {
        CyU3PDebugPrint (4, "UVC Create Event_0 failed, Error Code = %d\r\n", apiRetStatus);
        initErrors |= 1 << 8;
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Initialize Processor-Interface Block (PIB) for GPIF use. */
    pibclock.clkDiv      = 2;
    pibclock.clkSrc      = CY_U3P_SYS_CLK; /* 384 or 403.2 MHz clock depending on Device clock */
    pibclock.isDllEnable = CyFalse;        /* Used for async or master GPIF applications */
    pibclock.isHalfDiv   = CyFalse;        /* Adds 0.5 to value of .clkDiv */
    apiRetStatus = CyU3PPibInit(CyTrue, &pibclock);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "PIB Function Failed to Start, Error Code = 0x%x\n", apiRetStatus);
        initErrors |= 1 << 9;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Initialize the GPIF state machine. */
    CyFxApplnGpifInit(); /* It handles logging of its own errors, but the status is returned */

    CyU3PDebugPrint(8, "Initialized PIB and GPIF\r\n");

    /* Register the GPIF State Machine callback used to get frame end notifications.
     * We use the fast callback version which is triggered from ISR context.
     */
    CyU3PGpifRegisterSMIntrCallback(CyFxGpifCB);

    /* The fast enumeration is the easiest way to setup a USB connection,
     * where all enumeration phase is handled by the library. Only the
     * class / vendor requests need to be handled by the application. */
    CyU3PUsbRegisterSetupCallback(CyFxApplnUSBSetupCB, CyTrue);

    /* Setup the callback to handle the USB events */
    CyU3PUsbRegisterEventCallback(CyFxApplnUSBEventCB);

    /* Register a callback to handle LPM requests from the USB 3.0 host. */
    CyU3PUsbRegisterLPMRequestCallback(CyFxApplnLPMRqtCB);

    /* Set the USB Enumeration descriptors */

    /* Super speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB30DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set device descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 10;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB20DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set device descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 11;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* BOS descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0, (uint8_t *)CyFxUSBBOSDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set configuration descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 12;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Device qualifier descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0, (uint8_t *)CyFxUSBDeviceQualDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set device qualifier descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 13;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Super speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBSSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set configuration descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 14;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBHSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB Set Other Speed Descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 15;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Full speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBFSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB Set Configuration Descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 16;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 0 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxUSBStringLangIDDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set string descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 17;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 1 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBManufactureDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set string descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 18;
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 2 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(4, "USB set string descriptor failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 19;
        CyFxAppErrorHandler(apiRetStatus);
    }

    CyU3PDebugPrint(8, "Set callbacks and descriptors\r\n");

    /* Since the status interrupt endpoint is not used in this application,
     * just enable the EP in the beginning. */
    /* Control status interrupt endpoint configuration */
    epCfg.enable   = 1;
    epCfg.epType   = CY_U3P_USB_EP_INTR;
    epCfg.pcktSize = 64;
    epCfg.isoPkts  = 1;
    epCfg.burstLen = 1;

    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONTROL_STATUS, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 20;
        CyFxAppErrorHandler(apiRetStatus);
    }

#ifndef USE_DMA_OVERRIDE
    /* Create status interrupt DMA channel */
    dmaCfg.size = 16;
    dmaCfg.count = 0;
    dmaCfg.prodSckId = (CyU3PDmaSocketId_t) CY_U3P_CPU_SOCKET_PROD;
    dmaCfg.consSckId = (CyU3PDmaSocketId_t) (CY_U3P_UIB_SOCKET_CONS_0 | CY_FX_EP_CONTROL_SOCKET);
    dmaCfg.prodAvailCount = 0;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.notification = 0;
    dmaCfg.cb = 0;
    apiRetStatus = CyU3PDmaChannelCreate(&glChInterruptControl, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
    if(apiRetStatus != CY_U3P_SUCCESS)
    {
    CyU3PDebugPrint(2, "Status Channel creation failed with 0x%x\n", apiRetStatus);
    CyFxAppErrorHandler(apiRetStatus);
    }
#endif

    CyU3PDebugPrint(8, "Created USB EP0 Control endpoint\r\n");

    /* Connect the USB pins and enable super speed operation */
    apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB connect failed, Error code = 0x%x\r\n", apiRetStatus);
        initErrors |= 1 << 21;
        CyFxAppErrorHandler(apiRetStatus);
    }

    CyU3PDebugPrint(8, "Main application initialized\r\n");
}

/* Application define function which creates the threads. */
void
CyFxApplicationDefine (void)
{
    void *ptr1, *ptr2, *ptr3, *ptr4, *ptr5;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /** Thread Assignments:
     * ptr1 : UVC DMA Override Producer Thread
     *   Enabled with USB_UVC_CLASS && USE_DMA_OVERRIDE
     * ptr2 : UAC Streaming Thread
     *   Enabled with USB_UAC_CLASS
     * ptr3 : CDC Thread
     *   Enabled with CDC_UAC_CLASS
     * ptr4 : UVC Streaming Event Handler Thread
     *   Enabled with USB_UVC_CLASS && !USE_DMA_OVERRIDE
     * ptr5 : UVC EP0 Control Event Handler Thread
     *   Enabled with USB_UVC_CLASS && !USE_DMA_OVERRIDE
     */

    /* Allocate the memory for the thread stacks. */
    ptr1 = CyU3PMemAlloc(UVC_APP_THREAD_STACK); /* UVC DMA Producer */
    ptr2 = CyU3PMemAlloc(UVC_APP_THREAD_STACK); /* UVC Streaming */
    ptr3 = CyU3PMemAlloc(UVC_APP_THREAD_STACK); /* UVC Control */
    ptr4 = CyU3PMemAlloc(UVC_APP_THREAD_STACK); /* UAC Control */
    ptr5 = CyU3PMemAlloc(UVC_APP_THREAD_STACK); /* CDC Control */

    if ((ptr1 == 0) || (ptr2 == 0) || (ptr3 == 0) || (ptr4 == 0) || (ptr5 == 0))
        goto fatalMemError;
        
    /* Initialize the Debug Module */
    CyFxApplnDebugInit();                               /*add 20210818,configure uart*/


    /* Initialize the GPIO module. Used for timing pulse */
    CyFxApplnGpioInit();

    /* Print current configuration data */
    CyFxApplnPrintConf();

    /* Initialize the UVC Application */
    CyFxApplnInit();                                    /*initializes USB, creates event group,sets the enumeration descriptors, configures Endpoints and configures DMA*/

    /* Disable low power mode */
    CyU3PUsbLPMDisable();

#if USB_UVC_CLASS == 1

    retThrdCreate = CyU3PThreadCreate (&uvcAppThread,     /* UVC Thread structure */
                        "33:UVC_app_thread",              /* Thread Id and name */
                        UVCAppThread_Entry,               /* UVC Application Thread Entry function */
                        0,                                /* No input parameter to thread */
                        ptr4,                             /* Pointer to the allocated thread stack */
                        UVC_APP_THREAD_STACK,             /* UVC Application Thread stack size */
                        UVC_APP_THREAD_PRIORITY,          /* UVC Application Thread priority */
                        UVC_APP_THREAD_PRIORITY,          /* Pre-emption threshold */
                        CYU3P_NO_TIME_SLICE,              /* No time slice for the application thread */
                        CYU3P_AUTO_START                  /* Start the Thread immediately */
    );

    /* Check the return code */
    if (retThrdCreate != 0)
        goto fatalMemError;

    retThrdCreate = CyU3PThreadCreate (&uvcAppEp0Thread,  /* UVC Thread structure */
                        "34:UVC_ep0_thread",              /* Thread Id and name */
                        UVCAppEp0Thread_Entry,            /* UVC Application Thread Entry function */
                        0,                                /* No input parameter to thread */
                        ptr5,                             /* Pointer to the allocated thread stack */
                        UVC_APP_THREAD_STACK,             /* UVC Application Thread stack size */
                        UVC_APP_THREAD_PRIORITY,          /* UVC Application Thread priority */
                        UVC_APP_THREAD_PRIORITY,          /* Pre-emption threshold */
                        CYU3P_NO_TIME_SLICE,              /* No time slice for the application thread */
                        CYU3P_AUTO_START                  /* Start the Thread immediately */
    );

    /* Check the return code */
    if (retThrdCreate != 0)
        goto fatalMemError;

#if USE_DMA_OVERRIDE == 1
    /* Fill the manual test frames. For some reason I cannot put this into UVCAppProdThread_Entry */
    for (int i = 0; i < CY_FX_UVC_MAX_VID_FRAMES; i++){
        CyFxAppFillBuffer(&UvcdmaBuffer[i], (int)(128 / CY_FX_UVC_MAX_VID_FRAMES) * i);
        CyU3PDebugPrint(4, "Filled frame %d with %d\r\n", i, (int)(128 / CY_FX_UVC_MAX_VID_FRAMES * i));
    }

    retThrdCreate = CyU3PThreadCreate (&uvcAppProdThread, /* UVC Thread structure */
                        "30:UVC_dma_prod_thread",         /* Thread Id and name */
                        UVCAppProdThread_Entry,           /* UVC Application Thread Entry function */
                        0,                                /* No input parameter to thread */
                        ptr1,                             /* Pointer to the allocated thread stack */
                        UVC_APP_THREAD_STACK,             /* UVC Application Thread stack size */
                        UVC_APP_THREAD_PRIORITY,          /* UVC Application Thread priority */
                        UVC_APP_THREAD_PRIORITY,          /* Pre-emption threshold */
                        CYU3P_NO_TIME_SLICE,              /* No time slice for the application thread */
                        CYU3P_AUTO_START                  /* Start the Thread immediately */
    );

    /* Check the return code */
    if (retThrdCreate != 0)
        goto fatalMemError;
#endif /* USE_DMA_OVERRIDE */

#endif /* USB_UVC_CLASS */

#if USB_UAC_CLASS == 1

    /* Create 2 buffers manually for testing */
    UACDmaBuffer[0].buffer = CyU3PDmaBufferAlloc(96);
    UACDmaBuffer[1].buffer = CyU3PDmaBufferAlloc(96);

    CyU3PMemCopy(UACDmaBuffer[0].buffer, (uint8_t *)pcm_source_data, CY_FX3_ISO_XFER_LEN);
    CyU3PMemCopy(UACDmaBuffer[1].buffer, (uint8_t *)pcm_source_data + CY_FX3_ISO_XFER_LEN, CY_FX3_ISO_XFER_LEN);

    UACDmaBuffer[0].count = CY_FX3_ISO_XFER_LEN;
    UACDmaBuffer[0].size =  CY_FX3_ISO_XFER_LEN;
    UACDmaBuffer[0].status = 0;

    UACDmaBuffer[1].count = CY_FX3_ISO_XFER_LEN;
    UACDmaBuffer[1].size =  CY_FX3_ISO_XFER_LEN;
    UACDmaBuffer[1].status = 0;

    retThrdCreate = CyU3PThreadCreate (&uacAppThread,     /* UAC Thread structure */
                        "31:UAC_App_Thread",              /* Thread Id and name */
                        UACAppThread_Entry,               /* UAC Application Thread Entry function */
                        0,                                /* No input parameter to thread */
                        ptr2,                             /* Pointer to the allocated thread stack */
                        CY_FX_UAC_APP_THREAD_STACK,       /* UAC Application Thread stack size */
                        CY_FX_UAC_APP_THREAD_PRIORITY,    /* UAC Application Thread priority */
                        CY_FX_UAC_APP_THREAD_PRIORITY,    /* Pre-emption threshold */
                        CYU3P_NO_TIME_SLICE,              /* No time slice for the application thread */
                        CYU3P_AUTO_START                  /* Start the Thread immediately */
    );

    /* Check the return code */
    if (retThrdCreate != 0)
       goto fatalMemError;

#endif /* USB_UAC_CLASS */

#if USB_CDC_CLASS == 1

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&USBUARTAppThread, /* USBUART Example App Thread structure */
                        "32:UART_App_Thread",             /* Thread ID and Thread name */
                        USBUARTAppThread_Entry,           /* USBUART Example App Thread Entry function */
                        0,                                /* No input parameter to thread */
                        ptr3,                             /* Pointer to the allocated thread stack */
                        CY_FX_USBUART_THREAD_STACK,       /* USBUART Example App Thread stack size */
                        CY_FX_USBUART_THREAD_PRIORITY,    /* USBUART Example App Thread priority */
                        CY_FX_USBUART_THREAD_PRIORITY,    /* USBUART Example App Thread priority */
                        CYU3P_NO_TIME_SLICE,              /* No time slice for the application thread */
                        CYU3P_AUTO_START                  /* Start the Thread immediately */
    );

    /* Check the return code */
    if (retThrdCreate != 0)
        goto fatalMemError;

#endif /* USB_CDC_CLASS */

    return;

fatalMemError:

    /* Thread Creation failed with the error code retThrdCreate */

    /* Add custom recovery or debug actions here */

    /* Application cannot continue */
    /* Loop indefinitely */
    while(1);
}

/* Main function */
int
main (void)
{
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PSysClockConfig_t clockConfig;
    CyU3PReturnStatus_t   apiRetStatus = CY_U3P_SUCCESS;

    /* When the GPIF data bus is configured as 32-bits wide and running at
     * 100 MHz (synchronous), the FX3 system clock has to be set to a frequency
     * greater than 400 MHz.
     *
     * If set to CyFalse, an internal 384 MHz clock is used. If set to CyTrue,
     * a 403.2 MHz clock is used. To modify the GPIF interface clock, the input
     * clock config to the CyU3PPIBInit() function should be modified, using
     * this master clock as the 'CY_U3P_SYS_CLK' */
#if (GPIF_16BIT_CONF_SELECT == 0)
    clockConfig.setSysClk400  = CyTrue;
#else
    clockConfig.setSysClk400  = CyFalse;
#endif
    clockConfig.cpuClkDiv     = 2;
    clockConfig.dmaClkDiv     = 2;
    clockConfig.mmioClkDiv    = 2;
    clockConfig.useStandbyClk = CyFalse;
    clockConfig.clkSrc        = CY_U3P_SYS_CLK;
    /* Initialize the device */
    apiRetStatus = CyU3PDeviceInit (&clockConfig);        /*add 20210819,configure clock for cpu,DMA accesses,initialize VIC,only be called once and the first one*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable only the Instruction Cache. */
    apiRetStatus = CyU3PDeviceCacheControl (CyTrue, CyFalse, CyFalse);        /*add 20210819,both D-cache and DMAHandleDCache are disable*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix for the device. On the FX3 DVK board, the COM port 
     * is connected to the IO(53:56). This means that either DQ32 mode should be
     * selected or lppMode should be set to UART_ONLY. Here we are choosing
     * UART_ONLY configuration.
     * The isDQ32Bit, useUart, useI2C, useI2S, useSpi properties select the
     * pin configuration on the pin list (Table 7) of the datasheet. lppMode is
     * required to get UART properly hooked up in 16-bit mode.
     *
     * UART, I2C and I2C Enabled, CY_U3P_IO_MATRIX_LPP_DEFAULT w/ disabled 32bit
     * gives some log output
     * In 16-bit mode TRXS needs CY_U3P_IO_MATRIX_LPP_DEFAULT whereas dev board
     * needs CY_U3P_IO_MATRIX_LPP_UART_ONLY. Not sure why yet. */
    CyU3PMemSet ((uint8_t *)&io_cfg, 0, sizeof (io_cfg));
#if GPIF_16BIT_CONF_SELECT == 1
    io_cfg.isDQ32Bit        = CyFalse;                        /* 16-bit bus */
    io_cfg.useUart          = CyTrue;                         /* For logging */
    io_cfg.useI2C           = CyFalse;
    io_cfg.useI2S           = CyFalse;
    io_cfg.useSpi           = CyFalse;
    /* Choose pins for UART */
    io_cfg.lppMode          = CY_U3P_IO_MATRIX_LPP_DEFAULT;   /* Required for TRXS - GPIO[46]-GPIO[49] */
//    io_cfg.lppMode          = CY_U3P_IO_MATRIX_LPP_UART_ONLY; /* Required for dev board - GPIO[53]-GPIO[56] */
#else
    io_cfg.isDQ32Bit        = CyTrue;                         /* 32-bit bus */
    io_cfg.useUart          = CyTrue;                         /* For logging */
    io_cfg.useI2C           = CyFalse;                        /* Eventually for communicating host resolution requests */
    io_cfg.useI2S           = CyFalse;
    io_cfg.useSpi           = CyFalse;                        /* NOT AVAILABLE WITH 32-bit BUS */
    io_cfg.lppMode          = CY_U3P_IO_MATRIX_LPP_DEFAULT;   /* Required mode for 32-bit bus */
#endif
    io_cfg.s0Mode           = CY_U3P_SPORT_INACTIVE;          /* FX3 doesnt storage port */
    io_cfg.s1Mode           = CY_U3P_SPORT_INACTIVE;
    io_cfg.gpioSimpleEn[0]  = 0x00000000; /* GPIO[31]-GPIO[0]  */  /* No GPIOs are enabled. */
    io_cfg.gpioSimpleEn[1]  = 0x00000000; /* GPIO[63]-GPIO[32] */  /* One flag will act as clock pulse but will require CyU3PDeviceGpioOverride */
    io_cfg.gpioComplexEn[0] = 0x00000000; /* GPIO[22] is overridden as a GPIO output in CyFxApplnGPIOInit */
    io_cfg.gpioComplexEn[1] = 0x00000000;
    apiRetStatus = CyU3PDeviceConfigureIOMatrix (&io_cfg);          /*add,configure multiplexing IO*/
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();                                                    /*add 20210819,be invoked as the last function from the main () function*/

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:

    /* Cannot recover from this error. */
    while (1);
}

/* [ ] */

