/*
 ## Cypress USB 3.0 Platform header file (cyfxuvcinmem.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

#ifndef _INCLUDED_CYFXUVCINMEM_H_
#define _INCLUDED_CYFXUVCINMEM_H_

#include <cyu3externcstart.h>
#include <cyu3types.h>
#include <cyu3usbconst.h>

/* Functionality toggles */
#define USB_UVC_CLASS 1
#define USB_UAC_CLASS 0
#define USB_CDC_CLASS 0

/* Use DMA override to manually fill out DMA buffers and pass them to host
 * If set to 1, GPIF should produce buffers which are captured by DMA CB that
 * attached UVC headers and then commits packets to USB */
#define USE_DMA_OVERRIDE 0

/* Due to lack of UART access, the control endpoint can be used to send small
 * packets to the host unrequested. Only 1 packet seems to be able to be sent
 * per function call. May cause problems with USB driver and require the host
 * to reboot. Has no effect on execution. */
#define USE_INTERRUPT_LOGS 0

/* Select whether the GPIF data bus is 16 or 32 bits wide. When bus is set to
 * 16-bits UART logs become available for the TRXS.
 * If set to 1, 16 bits bus is selected. If set to 0, 32 bit bus is selected.
 * 32 bits is used for the primary application but due to design limitations
 * the 16 bit bus is required for UART debug prints. */
#define GPIF_16BIT_CONF_SELECT 1

/* Selector for 720p60 streaming. Should reduce bandwidth required over the
 * USB bus. If set to 1, the application will stream 720p60 video. Otherwise
 * 1080p60 will be streamed. */
#define STREAM_720P_SELECT 0

/* This header file comprises of the UVC application constants and
 * the video frame configurations */

/* Terminal/Unit IDs */
#define CY_FX_UVC_INPUT_TERMINAL_ID     (uint8_t)(1)    /* wIndex value used to select Camera terminal */
#define CY_FX_UVC_PROCESSING_UNIT_ID    (uint8_t)(2)    /* wIndex value used to select Processing Unit */
#define CY_FX_UVC_EXTENSION_UNIT_ID     (uint8_t)(3)    /* wIndex value used to select Extension Unit */
#define CY_FX_UVC_OUTPUT_TERMINAL_ID    (uint8_t)(4)    /* wIndex value used to select Output terminal */

#define UVC_APP_THREAD_STACK            (0x1000)        /* Thread stack size */
#define UVC_APP_THREAD_PRIORITY         (8)             /* Thread priority */

/* DMA Socket and USB Endpoint selection.
 * Used in determining which USB Endpoints and DMA Sockets are used
 * NOTE: USB applications cannot use EP0 b/c that is reserved for USB Control */
#define CY_FX_EP_BULK_SOCKET            0x01
#define CY_FX_EP_CONTROL_SOCKET         0x02 /* This is interrupt control for the UVC app, not USB Control (EP0) */

/* USB IN Endpoint definitions for UVC application */
#define CY_FX_EP_IN_TYPE                0x80            /* USB IN end points have MSB set */
#define CY_FX_EP_BULK_VIDEO             (CY_FX_EP_IN_TYPE | CY_FX_EP_BULK_SOCKET)    /* EP 1 IN configured as Bulk EP */
#define CY_FX_EP_CONTROL_STATUS         (CY_FX_EP_IN_TYPE | CY_FX_EP_CONTROL_SOCKET) /* EP 2 IN configured as Control EP */

/* DMA Socket definitions for UVC application
 * CY_U3P_UIB_SOCKET_CONS_0 is 0x300 indicating both the first USB consumer socket and coincidentally the
 * prefix for USB consumer sockets in general (0x30X) */
#define CY_FX_EP_VIDEO_CONS_SOCKET      (CY_U3P_UIB_SOCKET_CONS_0 | CY_FX_EP_BULK_SOCKET)
/* Socket which produces UVC data from FPGA */
#if USE_DMA_OVERRIDE == 1
#define CY_FX_EP_VIDEO_PROD_SOCKET      CY_U3P_CPU_SOCKET_PROD /* Linked by setting DMA buffer count to 0 and then manually inserting buffers into DMA channel */
#else
#define CY_FX_EP_VIDEO_PROD_SOCKET      CY_U3P_PIB_SOCKET_0    /* Linked with CyU3PGpifSocketConfigure */
#endif

/* UVC descriptor types */
#define CY_FX_INTF_ASSN_DSCR_TYPE       (11)                   /* Interface association descriptor type. */

#define CY_FX_UVC_MAX_VID_FRAMES        (8)                    /* Number of video frames for DMA override operation */

/* UVC VideoStreaming USB Endpoint packet Size
 * When modifying, make sure to alter the USB VideoStreaming Bulk Endpoint descriptor! */
#define CY_FX_EP_BULK_VIDEO_PKT_SIZE    (0x400)                /* 1024 bytes */

/* UVC VideoStreaming USB Endpoint packet Count
 * When modifying, make sure to alter the USB VideoStreaming Bulk Endpoint SuperSpeed Companion descriptor! */
#define CY_FX_EP_BULK_VIDEO_PKTS_COUNT  (16)                   /* 16 packets (burst size) per DMA buffer */

/* Maximum UVC Buffer size */
#define CY_FX_UVC_STREAM_BUF_SIZE       (CY_FX_EP_BULK_VIDEO_PKTS_COUNT * CY_FX_EP_BULK_VIDEO_PKT_SIZE)

/* Maximum usable UVC buffer size. Amount of video data that can be accommodated in one DMA buffer. */
#define CY_FX_UVC_BUF_FULL_SIZE         (CY_FX_UVC_STREAM_BUF_SIZE - 12 - 4) /* Account for header + footer */

/* UVC DMA Buffer count */
#define CY_FX_UVC_STREAM_BUF_COUNT      (4)

#define CY_FX_UVC_MAX_HEADER            (12)                   /* Maximum number of header bytes in UVC */
#define CY_FX_UVC_HEADER_DEFAULT_BFH    (0x8C)                 /* Default BFH(Bit Field Header) for the UVC Header */

#define CY_FX_UVC_MAX_PROBE_SETTING         (34)               /* Maximum number of bytes in Probe Control */
#define CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED (64)               /* Maximum number of bytes in Probe Control aligned to 32 byte */

#define CY_FX_UVC_HEADER_FRAME          (0)                    /* Normal frame indication */
#define CY_FX_UVC_HEADER_EOF            (uint8_t)(1 << 1)      /* End of frame indication */
#define CY_FX_UVC_HEADER_FRAME_ID       (uint8_t)(1 << 0)      /* Frame ID toggle bit */

#define CY_FX_UVC_INTERFACE_VC          (0)                    /* Video Control interface id. */
#define CY_FX_UVC_INTERFACE_VS          (1)                    /* Video Streaming interface id. */

#define CY_FX_USB_UVC_SET_REQ_TYPE      (uint8_t)(0x21)        /* UVC interface SET request type */
#define CY_FX_USB_UVC_GET_REQ_TYPE      (uint8_t)(0xA1)        /* UVC Interface GET request type */
#define CY_FX_USB_UVC_GET_CUR_REQ       (uint8_t)(0x81)        /* UVC GET_CUR request */
#define CY_FX_USB_UVC_SET_CUR_REQ       (uint8_t)(0x01)        /* UVC SET_CUR request */
#define CY_FX_USB_UVC_GET_DEF_REQ       (uint8_t)(0x87)        /* UVC GET_DEF request */
#define CY_FX_USB_UVC_GET_MIN_REQ       (uint8_t)(0x82)        /* UVC GET_MIN request */
#define CY_FX_USB_UVC_GET_MAX_REQ       (uint8_t)(0x83)        /* UVC GET_MAX request */
#define CY_FX_USB_UVC_GET_RES_REQ       (uint8_t)(0x84)        /* UVC GET_RES Request */
#define CY_FX_USB_UVC_GET_LEN_REQ       (uint8_t)(0x85)        /* UVC GET_LEN Request */
#define CY_FX_USB_UVC_GET_INFO_REQ      (uint8_t)(0x86)        /* UVC GET_INFO Request */
#define CY_FX_USB_UVC_GET_DEF_REQ       (uint8_t)(0x87)        /* UVC GET_DEF Request */

#define CY_FX_USB_UVC_VS_PROBE_CONTROL  (0x0100)               /* Control selector for VS_PROBE_CONTROL. */
#define CY_FX_USB_UVC_VS_COMMIT_CONTROL (0x0200)               /* Control selector for VS_COMMIT_CONTROL. */

/* Undefined Terminal Controls specific UVC control selector codes defined in the USB Video Class specification */
#define CY_FX_UVC_VC_REQUEST_ERROR_CODE_CONTROL             (uint16_t)(0x0200)

/* Video control Error Codes */
#define CY_FX_UVC_VC_ERROR_CODE_NO_ERROR                    (0x00)
#define CY_FX_UVC_VC_ERROR_CODE_NOT_READY                   (0x01)
#define CY_FX_UVC_VC_ERROR_CODE_WRONG_STATE                 (0x02)
#define CY_FX_UVC_VC_ERROR_CODE_POWER                       (0x03)
#define CY_FX_UVC_VC_ERROR_CODE_OUT_OF_RANGE                (0x04)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_UNIT                (0x05)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL             (0x06)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_REQUEST             (0x07)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_VAL_IN_RANGE        (0x08)
#define CY_FX_UVC_VC_ERROR_CODE_UNKNOWN                     (0xFF)

#define LOOP_TIMEOUT                                        (1000) /* Period of frame count updates. */

#define TIMING_PULSE_GPIO                                   (22)

/* Enum for a DMA reset event */
typedef enum CyFxUvcDmaResetVal
{
    CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE = 0,           /* FX3 DMA reset event haven't occurred */
    CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE,          /* FX3 DMA reset event caused due to a commit buffer failure */
    CY_FX_UVC_DMA_RESET_FRAME_TIMER_OVERFLOW            /* FX3 DMA reset event caused due to frame timer overflow */
} CyFxUvcDmaResetVal_t;

#define CY_FX_USB_SET_INTF_REQ_TYPE     (uint8_t)(0x01)         /* USB SET_INTERFACE Request Type. */
#define CY_FX_USB_SET_INTERFACE_REQ     (uint8_t)(0x0B)         /* USB SET_INTERFACE Request code. */

/* Maximum commit buffer failures to detect a stop streaming event in a MAC OS */
#define CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT    (30)

/* Event bits used for signaling the UVC application threads. */

/* Stream request event. Event flag that indicates that the streaming of video data has
   been enabled by the host. This flag is retained ON as long as video streaming is allowed,
   and is only turned off when the host indicates that data transfer should be stopped.
 */
#define CY_FX_UVC_STREAM_EVENT                  (1 << 0)

/* Abort streaming event. This event flag is set when the UVC host sends down a request
   (SET_INTERFACE or CLEAR_FEATURE) that indicates that video streaming should be stopped.
   The CY_FX_UVC_STREAM_EVENT event is cleared before setting this flag, and these two
   events can be considered as mutually exclusive.
 */
#define CY_FX_UVC_STREAM_ABORT_EVENT            (1 << 1)

/* UVC VIDEO_CONTROL_REQUEST event. This event flag indicates that a UVC class specific
   request addressed to the video control interface has been received. It should be cleared
   as soon as serviced by the firmware.
 */
#define CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT   (1 << 2)

/* UVC VIDEO_STREAM_REQUEST event. This event flag indicates that a UVC class specific
   request addressed to the video streaming interface has been received. It should be cleared
   as soon as serviced by the firmware.
 */
#define CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT    (1 << 3)

/* FX3 DMA Reset event. This event is set when FX3 is not able to commit a buffer due to a slower USB Host or due to
 * a frame timer overflow. When the device is streaming a higher resolution with higher fps, the USB bandwidth will
 * be saturated and Host will not be able to keep up. The video stream may work for few seconds and then device will
 * receive a commit buffer failure. It is also possible that Sensor/ISP fails to send video data due to some reasons.
 * In such cases, it is better to reset DMA and restart the video stream so that there is a continuous video preview.
 */
#define CY_FX_UVC_DMA_RESET_EVENT               (1 << 4)

/* USB suspend event handler. This event is set when the USB host sends a USB suspend event to put the FX3
 * device into low power mode. This event is sent when the Host application is closed and after the device enumerates.
 */
#define CY_FX_USB_SUSPEND_EVENT_HANDLER         (1 << 5)

/* DMA callback event. Set when the DMA callback is called by the DMA system due to a full buffer
 * or a partially complete buffer.
 */
#define CY_FX_UVC_DMA_CB                        (1 << 6)

/* Extern definitions of the USB Enumeration constant arrays used for the Application */
extern const uint8_t CyFxUSB20DeviceDscr[];
extern const uint8_t CyFxUSB30DeviceDscr[];
extern const uint8_t CyFxUSBDeviceQualDscr[];
extern const uint8_t CyFxUSBFSConfigDscr[];
extern const uint8_t CyFxUSBHSConfigDscr[];
extern const uint8_t CyFxUSBBOSDscr[];
extern const uint8_t CyFxUSBSSConfigDscr[];
extern const uint8_t CyFxUSBStringLangIDDscr[];
extern const uint8_t CyFxUSBManufactureDscr[];
extern const uint8_t CyFxUSBProductDscr[];

/* Extern definitions of the Video frame data */

/* UVC Probe Control Setting */
extern const uint8_t glProbeCtrl[CY_FX_UVC_MAX_PROBE_SETTING];
 
/* Video frame lengths */
extern const uint32_t glVidFrameLen[CY_FX_UVC_MAX_VID_FRAMES];

/* MJPEG Video Frames */
extern const uint8_t glUVCVidFrames[];

/* UAC PCM Frames */
extern const int16_t pcm_source_data[];

#include <cyu3externcend.h>


/*********************************************************************************************************/
/*********************************************UAC********************************************************/
/*********************************************************************************************************/

#define CY_FX_UAC_APP_THREAD_STACK     (0x1000)             /* Thread stack size */
#define CY_FX_UAC_APP_THREAD_PRIORITY  (8)                  /* Thread priority */

/* Endpoint definition for UAC application */
#define CY_FX_EP_ISO_AUDIO             (0x83)               /* EP 3 IN */

#define CY_FX_EP_AUDIO_CONS_SOCKET     (CY_U3P_UIB_SOCKET_CONS_0 | (CY_FX_EP_ISO_AUDIO & 0x7F)) /* Consumer socket 3 */

/* UAC streaming endpoint packet Count */
#define CY_FX_EP_ISO_AUDIO_PKTS_COUNT  (0x01)

/* UAC Buffer count */
#define CY_FX_UAC_STREAM_BUF_COUNT     (10)

#define CY_FX_UAC_STREAM_INTERFACE     (uint8_t)(1)         /* Streaming Interface : Alternate setting 1 */

#define CY_FX_UAC_INTERFACE_VC         (2)                  /* Audio Control interface id. */
#define CY_FX_UAC_INTERFACE_VS         (3)                  /* Audio Streaming interface id. */


/*********************************************************************************************************/
/*********************************************CDC********************************************************/
/*********************************************************************************************************/
#define  CY_FX_USBUART_DMA_BUF_COUNT    (1)
#define  CY_FX_USBUART_THREAD_STACK     (1000)
#define  CY_FX_USBUART_THREAD_PRIORITY  (8)

/* Endpoint and socket definitions for the USB-UART application */

#define CY_FX_EP_PRODUCER               0x05
#define CY_FX_EP_CONSUMER               0x85
#define CY_FX_EP_INTERRUPT              0x84

#define CY_FX_CDC_INTERFACE_CMD         (4)                      /* CDC CMD interface id. */
#define CY_FX_CDC_INTERFACE_DATA        (5)                      /* CDC DATA interface id. */

/* Descriptor Types */
#define CY_FX_BOS_DSCR_TYPE             15
#define CY_FX_DEVICE_CAPB_DSCR_TYPE     16
#define CY_FX_SS_EP_COMPN_DSCR_TYPE     48

/* Device Capability Type Codes */
#define CY_FX_WIRELESS_USB_CAPB_TYPE    1
#define CY_FX_USB2_EXTN_CAPB_TYPE       2
#define CY_FX_SS_USB_CAPB_TYPE          3
#define CY_FX_CONTAINER_ID_CAPBD_TYPE   4


#endif /* _INCLUDED_CYFXUVCINMEM_H_ */

/*[]*/

